package app;

import fuzzy.FuzzyNumber;
import fuzzyharmonysearch.applications.fuzzyspring.FuzzySpringHarmonyMemory;
import fuzzyharmonysearch.core.HarmonyMemory;
import fuzzyharmonysearch.core.ValueRange;
import hierarchicalinterpolation.control.Process;
import hierarchicalinterpolation.control.RuleBaseGenerator;
import hierarchicalinterpolation.model.Observation;
import hierarchicalinterpolation.model.Rule;
import hierarchicalinterpolation.model.SimpleTestFunction;
import hierarchicalinterpolation.model.TestFunction;
import originalharmonysearch.applications.beam.BeamHarmonyMemory;
import originalharmonysearch.applications.constraintfour.ConstraintFourHarmonyMemory;
import originalharmonysearch.applications.spring.SpringHarmonyMemory;
import originalharmonysearch.applications.vessel.VesselHarmonyMemory;
import org.apache.commons.math.stat.descriptive.DescriptiveStatistics;
import org.apache.commons.math.stat.inference.TTestImpl;
import util.GraphingData;
import weka.attributeSelection.*;
import weka.classifiers.Classifier;
import weka.classifiers.fuzzy.VQNN;
import weka.classifiers.meta.FeatureSelectionEnsemble;
import weka.core.Debug;
import weka.core.Instances;

import javax.swing.*;
import java.io.*;
import java.lang.Deprecated;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.Random;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicStampedReference;

import static java.util.Collections.sort;

/**
 * Created by IntelliJ IDEA.
 * User: rxd846
 * Date: Feb 1, 2010
 * Time: 2:24:55 AM
 * To change this template use File | Settings | File Templates.
 */
public class TestMain {

    private static double hierarchicalError = 0;
    private static double hierarchicalBackwardError = 0;
    private static double flatError = 0;

    private static void classifierAggregation(Classifier baseClassifier, int groupSize, String dataset) throws Exception {
        System.out.println("\nEvaluating Dataset - " + dataset);
        int numFolds = 10;

        Instances data;
        data = CrossValidation.openDataset(dataset);
        data.setClassIndex(data.numAttributes() - 1);

        Instances[] trainingFolds = new Instances[numFolds];
        Instances[] testingFolds = new Instances[numFolds];
        CrossValidation.generateCrossValidationFoldsGrouped(data, trainingFolds, testingFolds, groupSize);

        Classifier classifier;
        classifier = (Classifier) Class.forName(baseClassifier.getClass().getName()).newInstance();

        System.out.println(CrossValidation.crossValidateGrouped(classifier, trainingFolds, testingFolds, groupSize));


        //baseResult = CrossValidation.crossValidate(classifier, trainingTestingFolds);
        //System.out.println("Base\t" + classifier.getClass().getName());
    }

    public static void printRules(ArrayList<Rule> rules) {
        for (int i = 0; i < rules.size(); i++) {
            for (FuzzyNumber fuzzyNumber : rules.get(i).getAntecedents()) {
                System.out.print(fuzzyNumber.toString() + " ");
            }
            System.out.println(rules.get(i).getConsequence().toString());
        }
    }

    public static void printRules(Rule[] rules) {
        for (int i = 0; i < rules.length; i++) {
            for (FuzzyNumber fuzzyNumber : rules[i].getAntecedents()) {
                System.out.print(fuzzyNumber.toString() + " ");
            }
            System.out.println(rules[i].getConsequence().toString());
        }
    }

    public static void testInterpolation() {
        Process process = new hierarchicalinterpolation.control.Process();
        RuleBaseGenerator ruleBaseGenerator = new RuleBaseGenerator();
        //int numberOfVariables, double[] minAntecedentRanges, double[] maxAntecedentRanges, double minConsequenceRange, double maxConsequenceRange) {
        int numberOfVariables = 3;

        double[] ranges = new double[numberOfVariables];
        for (int i = 0; i < numberOfVariables; i++) {
            ranges[i] = 6;
        }

        double[] minAntecedentRanges = new double[numberOfVariables];
        double[] maxAntecedentRanges = new double[numberOfVariables];
        for (int i = 0; i < minAntecedentRanges.length; i++) {
            minAntecedentRanges[i] = 1;
            maxAntecedentRanges[i] = 15;
        }
        TestFunction testFunction = new SimpleTestFunction(numberOfVariables, minAntecedentRanges, maxAntecedentRanges,
                5, 25);
        ArrayList<Rule> rules = ruleBaseGenerator.generateRules(testFunction, 6);
        //printRules(rules);

        System.out.println(" ------------------------------------- ");

        Observation observation = ruleBaseGenerator.newObservation(numberOfVariables, ranges);
        double[] crispInput = ruleBaseGenerator.getCrispInputs(numberOfVariables, observation);
        System.out.println("Observation = " + observation.getAntecedents()[0].toString() + ", " +
                observation.getAntecedents()[1].toString() + ", " +
                observation.getAntecedents()[2].toString());
        double crispOutput = testFunction.calculate(crispInput);
        System.out.println("Function true output = " + crispOutput);

        System.out.println("---");

        try {
            FuzzyNumber conclusion = process.go(rules, observation);
            System.out.println("Hierarchical result = " + conclusion.toString());
            hierarchicalError = hierarchicalError + (conclusion.getRep() - crispOutput) * (conclusion.getRep() - crispOutput);

            System.out.println("---");

            FuzzyNumber conclusionBackward = process.goBackward(rules, observation);
            System.out.println("Hierarchical Backward result = " + conclusionBackward.toString());
            hierarchicalBackwardError = hierarchicalBackwardError + (conclusionBackward.getRep() - crispOutput) * (conclusionBackward.getRep() - crispOutput);

            System.out.println("---");

            FuzzyNumber conclusionFlat = process.goFlat(rules, observation);
            System.out.println("Flat averaging result = " + conclusionFlat.toString());
            flatError = flatError + (conclusionFlat.getRep() - crispOutput) * (conclusionFlat.getRep() - crispOutput);
        } catch (Exception e) {
            e.printStackTrace();  //TODO: Automatically Generated Catch Statement
        }

    }

    @Deprecated
    public static void testInterpolationNew() throws Exception {
        int numberOfVariables = 2;
        double[] minAntecedentRanges = new double[numberOfVariables];
        double[] maxAntecedentRanges = new double[numberOfVariables];
        for (int i = 0; i < minAntecedentRanges.length; i++) {
            minAntecedentRanges[i] = 0;
            maxAntecedentRanges[i] = 20;
        }

        Rule rule1 = new Rule(numberOfVariables, minAntecedentRanges, maxAntecedentRanges, 0, 10);
        rule1.getAntecedents()[0] = new FuzzyNumber(0, 4.5, 6);
        rule1.getAntecedents()[1] = new FuzzyNumber(11, 12.5, 14);
        rule1.setConsequence(new FuzzyNumber(0, 2.5, 4));

        Rule rule2 = new Rule(rule1);
        rule2.getAntecedents()[0] = new FuzzyNumber(12, 14.5, 16);
        rule2.getAntecedents()[1] = new FuzzyNumber(1, 2.5, 4);
        rule2.setConsequence(new FuzzyNumber(10, 11.5, 13));

        Observation observation = new Observation(rule1);
        observation.getAntecedents()[0] = new FuzzyNumber(3, 4, 5);
        observation.getAntecedents()[1] = new FuzzyNumber(6, 7, 8);
        observation.setConsequence(new FuzzyNumber(4.023847649653833,4.94798095730609,5.884421393040076));

        /*Rule rule1 = new Rule(numberOfVariables, minAntecedentRanges, maxAntecedentRanges, 0, 10);
        rule1.getAntecedents()[0] = new FuzzyNumber(0, 2.5, 4);
        rule1.getAntecedents()[1] = new FuzzyNumber(11, 12.5, 14);
        rule1.setConsequence(new FuzzyNumber(0, 4.5, 6));

        Rule rule2 = new Rule(rule1);
        rule2.getAntecedents()[0] = new FuzzyNumber(10, 11.5, 13);
        rule2.getAntecedents()[1] = new FuzzyNumber(1, 2.5, 4);
        rule2.setConsequence(new FuzzyNumber(12, 14.5, 16));

        Observation observation = new Observation(rule1);
        observation.getAntecedents()[0] = new FuzzyNumber(3.3094058443362675,5.760396516848938,8.673947638814797);
        observation.getAntecedents()[1] = new FuzzyNumber(6, 9, 12);
        observation.setConsequence(new FuzzyNumber(6, 8, 11));*/

        Rule intermediateRule = Process.getIntermediateRule(rule1, rule2, observation, 3);
        Rule transformedRule = Process.transform(intermediateRule, observation, 3);

        System.out.println(transformedRule.getConsequence().toString());

        intermediateRule = Process.getIntermediateRule(rule1, rule2, observation, 0);
        transformedRule = Process.transform(intermediateRule, observation, 0);
        System.out.println(transformedRule.getAntecedents()[0].toString());

        intermediateRule = Process.getIntermediateRule(rule1, rule2, observation, 1);
        transformedRule = Process.transform(intermediateRule, observation, 1);
        System.out.println(transformedRule.getAntecedents()[1].toString());
    }

    public static void main(String args[]) throws Exception {

        testInterpolationNew();

        /*int count = 0;
        while (count < 2) {
            testInterpolation();
            count++;
        }

        hierarchicalError = Math.sqrt(hierarchicalError) / count;
        hierarchicalBackwardError = Math.sqrt(hierarchicalBackwardError) / count;
        flatError = Math.sqrt(flatError) / count;

        System.out.println("Hierarchical Error S.D. = " + hierarchicalError);
        System.out.println("Hierarchical Backward Error S.D. = " + hierarchicalBackwardError);
        System.out.println("Flat Error S.D. = " + flatError);*/
        Process process = new hierarchicalinterpolation.control.Process();
        /*//Example 1
        FuzzyNumber AStar = new FuzzyNumber(7, 8, 9);
        FuzzyNumber a1 = new FuzzyNumber(0, 5, 6);
        FuzzyNumber a2 = new FuzzyNumber(11, 13, 14);
        FuzzyNumber b1 = new FuzzyNumber(0, 2, 4);
        FuzzyNumber b2 = new FuzzyNumber(10, 11, 13);
        */

        /*//Example 2
        FuzzyNumber AStar = new FuzzyNumber(5, 6, 8);
        FuzzyNumber a1 = new FuzzyNumber(3, 3, 3);
        FuzzyNumber a2 = new FuzzyNumber(12, 12, 12);
        FuzzyNumber b1 = new FuzzyNumber(4, 4, 4);
        FuzzyNumber b2 = new FuzzyNumber(10, 11, 13);*/

        //Example 1 Reverse
        FuzzyNumber AStar = new FuzzyNumber(5.83, 6.26, 7.38);
        FuzzyNumber b1 = new FuzzyNumber(0, 5, 6);
        FuzzyNumber b2 = new FuzzyNumber(11, 13, 14);
        FuzzyNumber a1 = new FuzzyNumber(0, 2, 4);
        FuzzyNumber a2 = new FuzzyNumber(10, 11, 13);

        double delta = Process.getDelta(AStar, a1, a2);
        FuzzyNumber ADash = Process.getIntermediateFuzzyTerm(a1, a2, delta);
        FuzzyNumber BDash = Process.getIntermediateFuzzyTerm(b1, b2, delta);
        //FuzzyNumber ADash = new FuzzyNumber(5.30, 8.85, 9.85);

        //FuzzyNumber BDash = new FuzzyNumber(4.81, 6.33, 8.33);
        System.out.println("AStar = " + AStar.toString() + " Rep = " + AStar.getRep());
        System.out.println("ADash = " + ADash.toString() + " Rep = " + ADash.getRep());
        System.out.println("BDash = " + BDash.toString() + " Rep = " + BDash.getRep());
        System.out.println("AStar Rep = " + AStar.getRep());
        double scaleRate = Process.getScaleRate(ADash, AStar);/*process.calculateScaleFactor(ADash, AStar);*/
        /*if (Double.isInfinite(scaleRate)) {
            double ratio = AStar.getSupportLength() / Math.abs(a1.getRep() - a2.getRep());
            double ratio2 = (AStar.getLeftShoulder() - a1.getRep()) / Math.abs(a1.getRep() - a2.getRep());
            double supportLength = ratio * Math.abs(b1.getRep() - b2.getRep());

            System.out.println("LR = " + (AStar.getCentroid() - AStar.getLeftShoulder()) / (AStar.getCentroid() - a1.getRep()));
            System.out.println("RR = " + (AStar.getRightShoulder() - AStar.getCentroid()) / (a2.getRep() - AStar.getCentroid()));

            double supposedLeft = ratio2 * Math.abs(b1.getRep() - b2.getRep()) + b1.getRep();
            ratio = supportLength / BDash.getSupportLength();
            System.out.println(supportLength + " <- " + BDash.getSupportLength());
            System.out.println("SL + " + supposedLeft + "BScaled = " + process.scale(BDash, ratio).toString() + " Rep = " + process.scale(BDash, ratio).getRep());
        }*/
        FuzzyNumber ADashScaled = Process.scale(ADash, scaleRate);/*process.scale(ADash, scaleFactor);*/
        double moveFactor = Process.getMoveRatio(ADashScaled, AStar);/*process.calculateMoveFactor(AScaled, AStar);*/
        FuzzyNumber AMoved = process.move(ADashScaled, moveFactor);
        System.out.println("AScaled = " + ADashScaled.toString());
        System.out.println("AMoved = " + process.move(process.scale(ADash, scaleRate), moveFactor).toString());
        System.out.println("BMoved = " + process.move(process.scale(BDash, scaleRate), moveFactor).toString());
        //System.out.println("BMovedNew = " + process.move(process.scaleNew(BDash, scaleFactor), moveFactor).toString());
        System.out.println("Scale = " + scaleRate + " - Move = " + moveFactor);

        //classifierAggregation(new VQNN(), 3, "COPD area data2_fs");

        /*double merits = 0;
        for (int i = 0; i < 10; i++) {
            merits += fuzzyOptimisationTest();
        }
        System.out.println("AVG = " + merits / 10);*/

        //testFeatureSelectionEnsemble(new FuzzyRoughNN());
        //testFeatureSelectionEnsemblePairedTTest(new FuzzyRoughNN(), 0, true);
        //testFeatureSelectionEnsemble(new VQNN());
        //testFeatureSelectionEnsembleMixed();
        //testFeatureSelectionEnsemble(new FuzzyRoughNN());


        /*int[][] m = new int[5][5];
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j ++) {
                m[i][j] = (new Random()).nextInt(10);
                System.out.print(m[i][j] + " ");
            }
            System.out.println();
        }
        printrr(m);*/

//        testIterativeSearch("CfsSubsetEval", 3000);
//        testIterativeSearch("FuzzyRoughSubsetEval", 2000);
//        testIterativeSearch("ConsistencySubsetEval", 3000);

        //testIterativeHarmonySearch("CfsSubsetEval", 5000, true, 5);
        //testIterativeHarmonySearch("FuzzyRoughSubsetEval", 2000, false, 5);


        //testHierarchicalHarmonySearch("CfsSubsetEval", 50000, false, 1);
        //testIterativeHarmonySearch("ConsistencySubsetEval", 3000, false, 25);
        /*testSearch("CfsSubsetEval", 3000, false, 25);
        testSearch("FuzzyRoughSubsetEval", 2000, false, 25);
        testSearch("ConsistencySubsetEval", 3000, false, 25);*/
        //testIterativeHarmonySearch("WrapperSubsetEval", 1000);

        /*JFrame f1 = new JFrame();
        f1.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        GraphingData graphingData1 = new GraphingData();
        graphingData1.setData(((PigsHarmonyMemory)featureSelectionHarmonyMemory).getHistoricalAcceptedValues()[0]);
        f1.add(graphingData1);
        f1.setSize(1000,1000);
        f1.setLocation(200,200);
        f1.setVisible(true);

        JFrame f2 = new JFrame();
        f2.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        GraphingData graphingData2 = ne w GraphingData();
        graphingData2.setData(((PigsHarmonyMemory)featureSelectionHarmonyMemory).getHistoricalAcceptedValues()[1]);
        f2.add(graphingData2);
        f2.setSize(800,800);
        f2.setLocation(200,200);
        f2.setVisible(true);*/


        //System.out.println(0 / 0);

        //ResultProcessor.processResults("artificial2_0.txt", "artificial2_1.txt", "artificial2_fnn.arff");
        //String result = ResultProcessor.readFile("artificial4_frnn2_s.arff", 0).toString();
        //System.out.println(result);
        //harmonySearch("sonar", "CfsSubsetEval", 1000, 10);
        //harmonySearch("sonar", "FuzzyRoughSubsetEval", 1000, 10);


        //testIterativeHarmonySearch("CfsSubsetEval", 3000);
        //testIterativeHarmonySearch("FuzzyRoughSubsetEval", 2000);
        //testIterativeHarmonySearch("ConsistencySubsetEval", 3000);
        //testIterativeHarmonySearch("WrapperSubsetEval", 1000);

        //iterativeSearch("heart", "FuzzyRoughSubsetEval", 1000);
        /*iterativeSearch("multifeat", "FuzzyRoughSubsetEval", 1000);
        iterativeSearch("multifeat", "FuzzyRoughSubsetEval", 1000);
        iterativeSearch("multifeat", "FuzzyRoughSubsetEval", 1000);
        iterativeSearch("multifeat", "FuzzyRoughSubsetEval", 1000);*/

        /*iterativeSearch("sonar", "CfsSubsetEval", 1000);
      iterativeSearch("sonar", "CfsSubsetEval", 1000);
      iterativeSearch("sonar", "CfsSubsetEval", 1000);
      iterativeSearch("sonar", "CfsSubsetEval", 1000);
      iterativeSearch("sonar", "CfsSubsetEval", 1000);*/

        //testFeatureSelectionEnsemble(new FuzzyRoughNN());

        //ResultFormatter.stripNumbers();
        //testFeatureSelectionEnsemble(new FuzzyRoughNN());
        //testFeatureSelectionEnsemble(new VQNN());
        //testFeatureSelectionEnsembleMixed();
        //testFeatureSelectionEnsemble(new FuzzyRoughNN());

        //vessel(originalharmonysearch.core.HarmonyMemory.createParameterRanges(20, 0.5, 0.8, 0.1), 1000, 5000);
        //vessel(originalharmonysearch.core.HarmonyMemory.createParameterRanges(20, 20, 0.8, 0.8, 0.85, 0.85, 0.001, 0.001), 100, 10000);
        //constraintFour(originalharmonysearch.core.HarmonyMemory.createParameterRanges(20, 0.5, 0.8, 0.1), 100, 5000);
        //spring(originalharmonysearch.core.HarmonyMemory.createParameterRanges(20, 0.5, 0.8, 0.1), 1000, 1000);
        //beam(originalharmonysearch.core.HarmonyMemory.createParameterRanges(20, 0.5, 0.8, 0.1), 1000, 10000);
        /*System.out.println("\nVar 20, 0.5, 0.8, 0.1");
        beam(originalharmonysearch.core.HarmonyMemory.createParameterRanges(20, 0.5, 0.8, 0.1), 100, 10000);
        System.out.println("\nVar 20, 0.5");
        beam2(100, 10000);
        System.out.println("\nFixed 20, 20, 0.8, 0.8, 0.85, 0.85, 0.001, 0.001");
        beam3(100, 10000);*/
    }

    public static double fuzzyOptimisationTest() throws Exception {
        System.out.println(FuzzyNumber.fuzzyLessThan(3.2, 3.1, 0.3));

        /*ValueRange[] valueRanges = new ValueRange[3];
        valueRanges[0] = new ValueRange(0, 20, true);
        for (int i = 1; i < valueRanges.length; i++) {
            valueRanges[i] = new ValueRange(0, 20, true);
        }

        PigsHarmonyMemory harmonyMemory =
                new PigsHarmonyMemory(
                        HarmonyMemory.createParameterRanges(20, 0.5),
                        valueRanges, new Random(new Random().nextLong()));*/

        /*SpringHarmonyMemory harmonyMemory =
                new SpringHarmonyMemory(
                        originalharmonysearch.core.HarmonyMemory.createParameterRanges(20, 0.5),
                        new originalharmonysearch.core.ValueRange[]{
                                new originalharmonysearch.core.ValueRange(0, 0.1, true),
                                new originalharmonysearch.core.ValueRange(0, 0.5, true),
                                new originalharmonysearch.core.ValueRange(8, 15, true)});

        harmonyMemory.fill();
        harmonyMemory.iterate(20000);

        System.out.println("Harmony: " + harmonyMemory.best().toString());
        System.out.println("Merit: " + harmonyMemory.best().getMerit());
        return harmonyMemory.best().getMerit();*/

        FuzzySpringHarmonyMemory harmonyMemory =
                new FuzzySpringHarmonyMemory(
                        HarmonyMemory.createParameterRanges(20, 0.5),
                        new ValueRange[]{new ValueRange(0, 0.1, true), new ValueRange(0, 0.5, true), new ValueRange(8, 15, true)});

        harmonyMemory.fill();
        harmonyMemory.iterate(5000);

        System.out.println("Harmony: " + harmonyMemory.best().toString());
        System.out.println("Merit: " + harmonyMemory.best().getMerit().toString() + " - " + harmonyMemory.best().getMerit().getCentreOfGravityX());
        return harmonyMemory.best().getMerit().getCentreOfGravityX();

        //pigsHarmonyMemory.printHarmonies();


        //System.out.println("Merit: " + harmonyMemory.best().getMerit());

        //((FuzzySpringHarmonyComparator) harmonyMemory.getHarmonyComparator()).checkConstraintVerbose(harmonyMemory.best());


        //plotGraph(((PigsHarmonyMemory) featureSelectionHarmonyMemory).getHistoricalAcceptedValues()[2]);
    }

    public static void plotGraph(double[] input) {
        JFrame f3 = new JFrame();
        f3.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        GraphingData graphingData3 = new GraphingData();
        graphingData3.setData(input);
        f3.add(graphingData3);
        f3.setSize(1200, 1000);
        f3.setLocation(200, 200);
        f3.setVisible(true);
    }

    public static void featureSelection(String evaluationMethod, String searchMethod, String databaseName) {
        ASEvaluation asEvaluation = null;
        ASSearch asSearch = null;
        Instances instances = null;
    }

    public static String computeCore(String databaseName, String evaluationMethod) throws Exception {
        Instances instances = null;
        try {
            instances = CrossValidation.openDataset(databaseName);
            instances.setClassIndex(instances.numAttributes() - 1);
        } catch (IOException e) {
            e.printStackTrace();
        }

        ASEvaluation asEvaluation = null;
        try {
            asEvaluation = (ASEvaluation) Class.forName("weka.attributeSelection." + evaluationMethod).newInstance();
            asEvaluation.buildEvaluator(instances);
            return ((FuzzyRoughSubsetEval) asEvaluation).computeCore().toString();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
        return "";
    }


    public static int greedyStepwise(String databaseName, String evaluationMethod) throws Exception {
        Instances instances = null;
        try {
            instances = CrossValidation.openDataset(databaseName);
            instances.setClassIndex(instances.numAttributes() - 1);
        } catch (IOException e) {
            e.printStackTrace();
        }

        ASEvaluation asEvaluation = null;
        try {
            asEvaluation = (ASEvaluation) Class.forName("weka.attributeSelection." + evaluationMethod).newInstance();
            asEvaluation.buildEvaluator(instances);
            System.out.println(((FuzzyRoughSubsetEval) asEvaluation).computeCore().toString());
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

        int size = instances.numAttributes();
        int[] searchResult = null;

        GreedyStepwise greedyStepwise = new GreedyStepwise();
        searchResult = greedyStepwise.search(asEvaluation, instances);

        BitSet bitSet = new BitSet(size);

        for (int i = 0; i < searchResult.length; i++) {
            bitSet.set(searchResult[i]);
        }

        for (int attribute : searchResult) System.out.print(attribute + " ");
        System.out.println("- (" + searchResult.length + ") - " + ((SubsetEvaluator) asEvaluation).evaluateSubset(bitSet));

        System.gc();

        return searchResult.length;
    }

    public static double fullAccuracy(
            String databaseName,
            String evaluationMethod,
            int maxIteration,
            boolean iterative) throws Exception {
        Instances instances = null;
        try {
            instances = CrossValidation.openDataset(databaseName);
            instances.setClassIndex(instances.numAttributes() - 1);
        } catch (IOException e) {
            e.printStackTrace();
        }

        System.gc();
        return CrossValidation.crossValidate(10, new VQNN(), instances);
        //return searchResult;
    }


    public static double iterativeHarmonySearch(
            String databaseName,
            String evaluationMethod,
            int maxIteration,
            boolean iterative) throws Exception {
        Instances instances = null;
        try {
            instances = CrossValidation.openDataset(databaseName);
            instances.setClassIndex(instances.numAttributes() - 1);
        } catch (IOException e) {
            e.printStackTrace();
        }

        ASEvaluation asEvaluation = null;
        try {
            asEvaluation = (ASEvaluation) Class.forName("weka.attributeSelection." + evaluationMethod).newInstance();
            asEvaluation.buildEvaluator(instances);
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

        int size = instances.numAttributes();
        //int[] searchResult = null;
        double merit = 0d;
        int[] searchResult = null;
        //HierarchicalHarmonySearch harmonySearch = new HierarchicalHarmonySearch();
        HarmonySearch harmonySearch = new HarmonySearch();
        harmonySearch.setIteration(maxIteration);
        harmonySearch.setNumMusicians(size);
        harmonySearch.search(asEvaluation, instances);
        System.out.print(size);
        if (iterative) {


            while ((harmonySearch.getM_bestMerit() >= merit)) {
                System.out.print("->" + harmonySearch.getSearchResult().length);
                if ((harmonySearch.getM_bestMerit() == merit) & (harmonySearch.getSearchResult().length == size)) break;

                size = harmonySearch.getSearchResult().length;
                merit = harmonySearch.getM_bestMerit();
                searchResult = harmonySearch.getSearchResult();
                harmonySearch.setNumMusicians(size);
                harmonySearch.search(asEvaluation, instances);

            }
            /*while (size > 0) {
                System.out.print(size + "->");
                harmonySearch.setIteration(maxIteration);
                harmonySearch.setNumMusicians(size);
                harmonySearch.search(asEvaluation, instances);

                if ((harmonySearch.getM_bestMerit() >= merit) & (harmonySearch.getSearchResult().length < size)) {
                    size = harmonySearch.getSearchResult().length;
                    merit = harmonySearch.getM_bestMerit();
                    searchResult = harmonySearch.getSearchResult();
                } else if (harmonySearch.getM_bestMerit() >= merit) {
                    merit = harmonySearch.getM_bestMerit();
                    searchResult = harmonySearch.getSearchResult();
                    break;
                } else {
                    break;
                }
            }*/
        } else {
            harmonySearch.setIteration(maxIteration);
            harmonySearch.setNumMusicians(size);
            harmonySearch.search(asEvaluation, instances);
            merit = harmonySearch.getM_bestMerit();
            searchResult = harmonySearch.getSearchResult();
        }

        BitSet bitSet = new BitSet(instances.numAttributes());

        for (int i = 0; i < searchResult.length; i++) {
            bitSet.set(searchResult[i]);
        }
        System.out.print("\t");
        for (int attribute : searchResult) System.out.print(attribute + " ");
        System.out.println("\t" + searchResult.length + "\t" + merit);

        Classifier classifier = CrossValidation.filterClassifier(new VQNN(), searchResult, instances.classIndex());
        System.gc();
        return CrossValidation.crossValidate(10, classifier, instances);
        //return searchResult;
    }

    public static void hierarchicalHarmonySearch(
            String databaseName,
            String evaluationMethod,
            int maxIteration,
            boolean iterative) throws Exception {
        Instances instances = null;
        try {
            instances = CrossValidation.openDataset(databaseName);
            instances.setClassIndex(instances.numAttributes() - 1);
        } catch (IOException e) {
            e.printStackTrace();
        }

        ASEvaluation asEvaluation = null;
        try {
            asEvaluation = (ASEvaluation) Class.forName("weka.attributeSelection." + evaluationMethod).newInstance();
            asEvaluation.buildEvaluator(instances);
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

        int size = instances.numAttributes();
        //int[] searchResult = null;
        double merit = 0d;
        int[] searchResult = null;
        //HierarchicalHarmonySearch harmonySearch = new HierarchicalHarmonySearch();
        HierarchicalHarmonySearch harmonySearch = new HierarchicalHarmonySearch();

        if (iterative) {
            while (size > 0) {
                System.out.print(size + "->");
                harmonySearch.setIteration(maxIteration);
                harmonySearch.setNumMusicians(size);
                harmonySearch.search(asEvaluation, instances);

                if ((harmonySearch.getM_bestMerit() >= merit) & (harmonySearch.getSearchResult().length < size)) {
                    size = harmonySearch.getSearchResult().length;
                    merit = harmonySearch.getM_bestMerit();
                    searchResult = harmonySearch.getSearchResult();
                } else if (harmonySearch.getM_bestMerit() >= merit) {
                    merit = harmonySearch.getM_bestMerit();
                    searchResult = harmonySearch.getSearchResult();
                    break;
                } else {
                    break;
                }
            }
        } else {
            harmonySearch.setIteration(maxIteration);
            harmonySearch.setNumMusicians(size);
            harmonySearch.search(asEvaluation, instances);
            merit = harmonySearch.getM_bestMerit();
            searchResult = harmonySearch.getSearchResult();
        }

        BitSet bitSet = new BitSet(instances.numAttributes());

        for (int i = 0; i < searchResult.length; i++) {
            bitSet.set(searchResult[i]);
        }
        System.out.print("\t");
        for (int attribute : searchResult) System.out.print(attribute + " ");
        System.out.println("\t" + searchResult.length + "\t" + merit);

        System.gc();
    }

    public static void iterativeSearch(String databaseName, String evaluationMethod, int maxIteration) throws Exception {
        Instances instances = null;
        try {
            instances = CrossValidation.openDataset(databaseName);
            instances.setClassIndex(instances.numAttributes() - 1);
        } catch (IOException e) {
            e.printStackTrace();
        }

        ASEvaluation asEvaluation = null;
        try {
            asEvaluation = (ASEvaluation) Class.forName("weka.attributeSelection." + evaluationMethod).newInstance();
            asEvaluation.buildEvaluator(instances);
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

        int size = instances.numAttributes();
        //int[] searchResult = null;
        double merit = 0d;
        int[] searchResult = null;
        //HierarchicalHarmonySearch harmonySearch = new HierarchicalHarmonySearch();
        PSOSearch psoSearch = new PSOSearch();
        //GreedyStepwise greedyStepwise = new GreedyStepwise();
        //greedyStepwise.setSearchBackwards(true);
        /*while (size > 0) {
            System.out.print(size + "->");
            harmonySearch.setIteration(maxIteration);
            harmonySearch.setNumMusicians(size);
            harmonySearch.search(asEvaluation, instances);

            if ((harmonySearch.getM_bestMerit() >= merit) & (harmonySearch.getSearchResult().length < size)) {
                size = harmonySearch.getSearchResult().length;
                merit = harmonySearch.getM_bestMerit();
                searchResult = harmonySearch.getSearchResult();
            } else if (harmonySearch.getM_bestMerit() >= merit) {
                merit = harmonySearch.getM_bestMerit();
                searchResult = harmonySearch.getSearchResult();
                break;
            } else {
                break;
            }
        }*/
        searchResult = psoSearch.search(asEvaluation, instances);
        //merit = hillClimber.getM_bestMerit();
        BitSet bitSet = new BitSet(instances.numAttributes());

        for (int i = 0; i < searchResult.length; i++) {
            bitSet.set(searchResult[i]);
        }

        for (int attribute : searchResult) System.out.print(attribute + " ");
        System.out.println("\t" + searchResult.length + "\t" + ((SubsetEvaluator) asEvaluation).evaluateSubset(bitSet));

        System.gc();
    }


    public static void iterativeHarmonySearch(String databaseName, String evaluationMethod, int maxIteration, int musicianSize) throws Exception {
        Instances instances = null;
        try {
            instances = CrossValidation.openDataset(databaseName);
            instances.setClassIndex(instances.numAttributes() - 1);
        } catch (IOException e) {
            e.printStackTrace();
        }

        ASEvaluation asEvaluation = null;
        try {
            asEvaluation = (ASEvaluation) Class.forName("weka.attributeSelection." + evaluationMethod).newInstance();
            asEvaluation.buildEvaluator(instances);
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

        int size = musicianSize;
        //int[] searchResult = null;
        double merit = 0d;
        HarmonySearch harmonySearch = new HarmonySearch();

        while (size > 0) {

            harmonySearch.setIteration(maxIteration);
            harmonySearch.setNumMusicians(size);
            harmonySearch.search(asEvaluation, instances);

            if ((harmonySearch.getM_bestMerit() >= merit) && (harmonySearch.getSearchResult().length < size)) {
                size = harmonySearch.getSearchResult().length;
                merit = harmonySearch.getM_bestMerit();
            } else {
                break;
            }
        }

        BitSet bitSet = new BitSet(instances.numAttributes());

        for (int i = 0; i < harmonySearch.getSearchResult().length; i++) {
            bitSet.set(harmonySearch.getSearchResult()[i]);
        }

        for (int attribute : harmonySearch.getSearchResult()) System.out.print(attribute + " ");
        System.out.println("- (" + harmonySearch.getSearchResult().length + ") - " + ((SubsetEvaluator) asEvaluation).evaluateSubset(bitSet));

        System.gc();
    }

    public static void harmonySearch(String databaseName, String evaluationMethod, int maxIteration, int ensembleSize) throws Exception {

        Instances instances = null;
        try {
            instances = CrossValidation.openDataset(databaseName);
            instances.setClassIndex(instances.numAttributes() - 1);
        } catch (IOException e) {
            e.printStackTrace();
        }

        ASEvaluation asEvaluation = null;
        try {
            asEvaluation = (ASEvaluation) Class.forName("weka.attributeSelection." + evaluationMethod).newInstance();
            asEvaluation.buildEvaluator(instances);
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

        HarmonySearch harmonySearch = new HarmonySearch();
        harmonySearch.setIteration(maxIteration);
        harmonySearch.setNumMusicians(ensembleSize);

        harmonySearch.search(asEvaluation, instances);
    }

    public static ArrayList<Classifier[]> buildBaseClassifiers(
            Classifier baseClassifier,
            int size,
            int poolMethod,
            Instances[] trainingTestingFolds
    ) throws Exception {
        ArrayList<Classifier[]> baseClassifiers = new ArrayList<Classifier[]>();

        FeatureSelectionEnsemble classifier = new FeatureSelectionEnsemble();
        for (int i = 0; i < trainingTestingFolds.length / 2; i++) {
            baseClassifiers.add(classifier.generateBaseClassifiers(baseClassifier, size, poolMethod, trainingTestingFolds[i]));
        }
        return baseClassifiers;
    }

    public static Classifier[] buildFeatureSelectionEnsembles(
            Classifier baseClassifier,
            int size,
            int poolMethod,
            Instances[] trainingTestingFolds) throws Exception {

        Classifier[] prebuiltClassifiers = new Classifier[trainingTestingFolds.length / 2];

        for (int i = 0; i < trainingTestingFolds.length / 2; i++) {
            FeatureSelectionEnsemble featureSelectionEnsemble = new FeatureSelectionEnsemble();
            featureSelectionEnsemble.setBaseClassifiers(
                    featureSelectionEnsemble.generateBaseClassifiers(baseClassifier, size, poolMethod, trainingTestingFolds[i]));
            featureSelectionEnsemble.buildDecisionMatrix(trainingTestingFolds[i]);
            prebuiltClassifiers[i] = featureSelectionEnsemble;
        }
        System.gc();
        return prebuiltClassifiers;
    }

    public static void testIterativeHarmonySearch(
            String databaseName,
            String evaluationMethod,
            int maxIteration,
            boolean iterative,
            int repeat) throws Exception {
        System.out.println("Current Database: " + databaseName);
        System.out.println("Full Accuracy = " + fullAccuracy(databaseName, evaluationMethod, maxIteration, iterative));
        double accuracy = 0;
        for (int i = 0; i < repeat; i++) {
            accuracy += iterativeHarmonySearch(databaseName, evaluationMethod, maxIteration, iterative);
        }
        System.out.println("Average Accuracy = " + accuracy / repeat + "");
    }

    public static void testHierarchicalHarmonySearch(
            String databaseName,
            String evaluationMethod,
            int maxIteration,
            boolean iterative,
            int repeat) throws Exception {
        System.out.println("Current Database: " + databaseName);
        for (int i = 0; i < repeat; i++) {
            hierarchicalHarmonySearch(databaseName, evaluationMethod, maxIteration, iterative);
        }
    }

    public static void testSearch(
            String databaseName,
            String evaluationMethod,
            int maxIteration,
            boolean iterative,
            int repeat) throws Exception {
        System.out.println("Current Database: " + databaseName);
        for (int i = 0; i < repeat; i++) {
            iterativeSearch(databaseName, evaluationMethod, maxIteration);
        }
    }

    public static void testIterativeSearch(String databaseName, String evaluationMethod, int maxIteration) throws Exception {
        System.out.println("Current Database: " + databaseName);
        //fullSearch(databaseName);
        //computeCore(databaseName, evaluationMethod);
        //int musicianSize = greedyStepwise(databaseName, evaluationMethod);
        iterativeSearch(databaseName, evaluationMethod, maxIteration);
        iterativeSearch(databaseName, evaluationMethod, maxIteration);
        iterativeSearch(databaseName, evaluationMethod, maxIteration);
        iterativeSearch(databaseName, evaluationMethod, maxIteration);
        iterativeSearch(databaseName, evaluationMethod, maxIteration);
    }

    public static void fullSearch(String databaseName) {
        Instances instances = null;
        try {
            instances = CrossValidation.openDataset(databaseName);
            instances.setClassIndex(instances.numAttributes() - 1);
        } catch (IOException e) {
            e.printStackTrace();
        }

        FuzzyRoughSubsetEval fuzzyRoughSubsetEval = new FuzzyRoughSubsetEval();
        try {
            fuzzyRoughSubsetEval.buildEvaluator(instances);
        } catch (Exception e) {
            e.printStackTrace();  //TODO: Automatically Generated Catch Statement
        }

        try {
            (new FullReductSearch()).search(fuzzyRoughSubsetEval, instances);
        } catch (Exception e) {
            e.printStackTrace();  //TODO: Automatically Generated Catch Statement
        }
    }

    public static void testIterativeHarmonySearch(String evaluationMethod, int maxIteration, boolean iterative, int repeat) throws Exception {
        for (int i = 6; i < 8; i++) {
            testIterativeHarmonySearch(selectDatabase(i), evaluationMethod, maxIteration, iterative, repeat);
        }
    }

    public static void testHierarchicalHarmonySearch(String evaluationMethod, int maxIteration, boolean iterative, int repeat) throws Exception {
        for (int i = 11; i < 12; i++) {
            testHierarchicalHarmonySearch(selectDatabase(i), evaluationMethod, maxIteration, iterative, repeat);
        }
    }

    public static void testSearch(String evaluationMethod, int maxIteration, boolean iterative, int repeat) throws Exception {
        for (int i = 9; i < 11; i++) {
            testSearch(selectDatabase(i), evaluationMethod, maxIteration, iterative, repeat);
        }
    }

    public static String selectDatabase(int index) {
        switch (index) {
            case 0:
                return "cleveland";
            case 1:
                return "ecoli";
            case 2:
                return "glassScaled";
            case 3:
                return "heart";
            case 4:
                return "ionosphere";
            case 5:
                return "sonar";
            case 6:
                return "2-completed";
            case 7:
                return "3-completed";
            case 8:
                return "wineScaled";
            case 9:
                return "olitos";
            case 10:
                return "arrhythmia";
            case 13:
                return "web";
            case 14:
                return "waveform-+noise";
            case 15:
                return "multifeat";
            case 16:
                return "wisconsin";
            default:
                return "";

        }
    }

    public static void testIterativeSearch(String evaluationMethod, int maxIteration) throws Exception {
        /*crossValidateFeatureSelectionEnsemble("2", baseC);
       crossValidateFeatureSelectionEnsemble("3", baseC);*/
        String currentDatabase;
        /*currentDatabase = "wisconsin";
      testIterativeHarmonySearch(currentDatabase, evaluationMethod, maxIteration);*/
        currentDatabase = "cleveland";
        testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);
        currentDatabase = "ecoli";
        testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);
        currentDatabase = "glassScaled";
        testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);
        currentDatabase = "heart";
        testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);
        currentDatabase = "ionosphere";
        testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);
        currentDatabase = "sonar";
        testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);
        currentDatabase = "2";
        testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);
        currentDatabase = "3";
        testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);
        /*currentDatabase = "waveform-+noise";
      testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);*/
        currentDatabase = "wineScaled";
        testIterativeSearch(currentDatabase, evaluationMethod, maxIteration);
    }

    public static void testFeatureSelectionEnsemblePairedTTest(Classifier baseC, int poolMethod, boolean isSupervised) throws Exception {
        /*crossValidateFeatureSelectionEnsemble("2", baseC);
        crossValidateFeatureSelectionEnsemble("3", baseC);*/
        //crossValidateFeatureSelectionEnsemble("waveform-+noise", baseC);
        crossValidateFeatureSelectionEnsemblePairedTTest("cleveland", baseC, poolMethod, isSupervised);
        crossValidateFeatureSelectionEnsemblePairedTTest("ecoli", baseC, poolMethod, isSupervised);
        crossValidateFeatureSelectionEnsemblePairedTTest("glassScaled", baseC, poolMethod, isSupervised);
        crossValidateFeatureSelectionEnsemblePairedTTest("heart", baseC, poolMethod, isSupervised);
        crossValidateFeatureSelectionEnsemblePairedTTest("ionosphere", baseC, poolMethod, isSupervised);
        crossValidateFeatureSelectionEnsemblePairedTTest("sonar", baseC, poolMethod, isSupervised);
        crossValidateFeatureSelectionEnsemblePairedTTest("3-completed", baseC, poolMethod, isSupervised);
        crossValidateFeatureSelectionEnsemblePairedTTest("2-completed", baseC, poolMethod, isSupervised);
        crossValidateFeatureSelectionEnsemblePairedTTest("wineScaled", baseC, poolMethod, isSupervised);
        //crossValidateFeatureSelectionEnsemble("2", baseC);
    }

    public static void testFeatureSelectionEnsemble(Classifier baseC) throws Exception {
        /*crossValidateFeatureSelectionEnsemble("2", baseC);
        crossValidateFeatureSelectionEnsemble("3", baseC);*/
        //crossValidateFeatureSelectionEnsemble("waveform-+noise", baseC);
        //crossValidateFeatureSelectionEnsemble("cleveland", baseC);
        crossValidateFeatureSelectionEnsemble("wisconsin", baseC);
        //crossValidateFeatureSelectionEnsemble("glassScaled", baseC);
        /*crossValidateFeatureSelectionEnsemble("heart", baseC);
        crossValidateFeatureSelectionEnsemble("ionosphere", baseC);
        crossValidateFeatureSelectionEnsemble("sonar", baseC);
        crossValidateFeatureSelectionEnsemble("3-completed", baseC);
        crossValidateFeatureSelectionEnsemble("2-completed", baseC);
        crossValidateFeatureSelectionEnsemble("wineScaled", baseC);*/
        //crossValidateFeatureSelectionEnsemble("2", baseC);
    }

    public static void testFeatureSelectionEnsembleMixed() throws Exception {
        /*crossValidateFeatureSelectionEnsembleMixedBagging("sonar");
        crossValidateFeatureSelectionEnsembleMixedBagging("wineScaled");
        crossValidateFeatureSelectionEnsembleMixedBagging("heart");
        //crossValidateFeatureSelectionEnsembleMixedBagging("web");
        //crossValidateFeatureSelectionEnsembleMixedBagging("arrhythmia");
        crossValidateFeatureSelectionEnsembleMixedBagging("ecoli");
        crossValidateFeatureSelectionEnsembleMixedBagging("glassScaled");
        crossValidateFeatureSelectionEnsembleMixedBagging("3-completed");
        crossValidateFeatureSelectionEnsembleMixedBagging("2-completed");
        //crossValidateFeatureSelectionEnsembleMixedBagging("waveform-+noise");
        crossValidateFeatureSelectionEnsembleMixedBagging("ionosphere");
        crossValidateFeatureSelectionEnsembleMixedBagging("cleveland");*/

        crossValidateFeatureSelectionEnsembleMixed("sonar");
        crossValidateFeatureSelectionEnsembleMixed("wineScaled");
        crossValidateFeatureSelectionEnsembleMixed("heart");
        //crossValidateFeatureSelectionEnsembleMixed("web");
        //crossValidateFeatureSelectionEnsembleMixed("arrhythmia");
        crossValidateFeatureSelectionEnsembleMixed("ecoli");
        crossValidateFeatureSelectionEnsembleMixed("glassScaled");
        crossValidateFeatureSelectionEnsembleMixed("3-completed");
        crossValidateFeatureSelectionEnsembleMixed("2-completed");
        //crossValidateFeatureSelectionEnsembleMixed("waveform-+noise");
        crossValidateFeatureSelectionEnsembleMixed("ionosphere");
        crossValidateFeatureSelectionEnsembleMixed("cleveland");
    }

    /*public static void crossValidateFeatureSelectionEnsemble(String dataset, Classifier baseC) throws Exception {
        System.out.println("\nEvaluating Dataset - " + dataset);
        Instances data;
        Classifier classifier;
        double result;

        data = CrossValidation.openDataset(dataset);
        data.setClassIndex(data.numAttributes() - 1);

        classifier = (Classifier) Class.forName(baseC.getClass().getName()).newInstance();
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("Base\t" + classifier.getClass().getName() + "\t\t" + result);

        *//*classifier = new Bagging();
        ((Bagging) classifier).setClassifier(new FuzzyRoughNN());
        //((Bagging) classifier).setOptions();
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("Bagging " + result);*//*

        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setBaseClassifier(baseC);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(121354);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\tRandom\tBagging\t" + result);

        //FeatureSelection + 999 + Partition
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setBaseClassifier(baseC);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(999);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t999\tBagging\t" + result);

        //FeatureSelection + 111 + Partition
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setBaseClassifier(baseC);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t111\tBagging\t" + result);

        //FeatureSelection + 5 + Partition
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setBaseClassifier(baseC);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(5);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t5\tBagging\t" + result);

        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setBaseClassifier(baseC);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(121354);
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(1);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\tRandom\tPartition\t" + result);

        //FeatureSelection + 999 + Bagging
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setBaseClassifier(baseC);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(999);
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(1);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t999\tPartition\t" + result);

        //FeatureSelection + 111 + Bagging
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setBaseClassifier(baseC);
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(1);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t111\tPartition\t" + result);

        //FeatureSelection + 5 + Bagging
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setBaseClassifier(baseC);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(5);
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(1);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t5\tPartition\t" + result);
    }*/

    public static void crossValidateFeatureSelectionEnsemblePairedTTest(String dataset, Classifier baseC, int poolMethod, boolean isSupervised) throws Exception {
        System.out.println("\nEvaluating Dataset - " + dataset);
        int numFolds = 10;

        Instances data;
        data = CrossValidation.openDataset(dataset);
        data.setClassIndex(data.numAttributes() - 1);

        double result;
        double baseResult;
        Instances[] trainingTestingFolds = CrossValidation.generateCrossValidationFolds(numFolds, data);

        Classifier classifier;
        classifier = (Classifier) Class.forName(baseC.getClass().getName()).newInstance();
        baseResult = CrossValidation.crossValidate(classifier, trainingTestingFolds);
        System.out.println("Base\t" + classifier.getClass().getName());

        Classifier[] prebuiltClassifiers;
        //int poolMethod;
        int size = 50;

        //poolMethod = 0; //Bagging;
        prebuiltClassifiers = buildFeatureSelectionEnsembles(baseC, size, poolMethod, trainingTestingFolds);

        System.out.println("FeatureSelection\t" + (isSupervised ? "111" : "5") + "\t" + (poolMethod == 0 ? "Bagging" : "Partition"));
        for (int i = 0; i < 10; i++) {
            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 999, trainingTestingFolds);
            System.out.print(result);
            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, isSupervised ? 111 : 5, trainingTestingFolds);
            System.out.println("\t" + result);
        }

        /*System.out.println("FeatureSelection\t5\tBagging");
        for (int i = 0; i < 50; i++) {
            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 5, trainingTestingFolds);
            System.out.println(baseResult + "\t" + result);
        }*/

        //poolMethod = 1;
        /*prebuiltClassifiers = buildFeatureSelectionEnsembles(baseC, size, poolMethod, trainingTestingFolds);

        System.out.println("FeatureSelection\t111\tPartition");
        for (int i = 0; i < 50; i++) {
            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 111, trainingTestingFolds);
            System.out.println(baseResult + "\t" + result);
        }

        System.out.println("FeatureSelection\t5\tPartition");
        for (int i = 0; i < 50; i++) {
            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 5, trainingTestingFolds);
            System.out.println(baseResult + "\t" + result);
        }*/

        System.gc();
    }


    public static void crossValidateFeatureSelectionEnsemble(String dataset, Classifier baseC) throws Exception {
        System.out.println("\nEvaluating Dataset - " + dataset);
        int numFolds = 10;

        Instances data;
        data = CrossValidation.openDataset(dataset);
        data.setClassIndex(data.numAttributes() - 1);

        double result;
        Instances[] trainingTestingFolds = CrossValidation.generateCrossValidationFolds(numFolds, data);

        Classifier classifier;
        classifier = (Classifier) Class.forName(baseC.getClass().getName()).newInstance();
        result = CrossValidation.crossValidate(classifier, trainingTestingFolds);
        System.out.println("Base\t" + classifier.getClass().getName() + "\t\t" + result);

        Classifier[] prebuiltClassifiers;
        int poolMethod;
        int size = 50;

        System.out.println("Bagging\tFull\tRandom\tFRFS\tU-FRFS");

        for (int i = 0; i < 10; i++) {
            poolMethod = 0; //Bagging;
            prebuiltClassifiers = buildFeatureSelectionEnsembles(baseC, size, poolMethod, trainingTestingFolds);

            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 999, trainingTestingFolds);
            System.out.print("\t" + result);

            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 121354, trainingTestingFolds);
            System.out.print("\t" + result);

            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 111, trainingTestingFolds);
            System.out.print("\t" + result);

            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 5, trainingTestingFolds);
            System.out.print("\t" + result + "\n");
        }

        System.out.println("Partition\tFull\tRandom\tFRFS\tU-FRFS");

        for (int i = 0; i < 10; i++) {
            poolMethod = 1;
            prebuiltClassifiers = buildFeatureSelectionEnsembles(baseC, size, poolMethod, trainingTestingFolds);

            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 999, trainingTestingFolds);
            System.out.print("\t" + result);

            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 121354, trainingTestingFolds);
            System.out.print("\t" + result);

            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 111, trainingTestingFolds);
            System.out.print("\t" + result);

            result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 5, trainingTestingFolds);
            System.out.print("\t" + result + "\n");
        }

        /*poolMethod = 0; //Bagging;
        prebuiltClassifiers = buildFeatureSelectionEnsembles(baseC, size, poolMethod, trainingTestingFolds);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 999, trainingTestingFolds);
        System.out.println("FeatureSelection\t999\tBagging\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 121354, trainingTestingFolds);
        System.out.println("FeatureSelection\tRandom\tBagging\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 111, trainingTestingFolds);
        System.out.println("FeatureSelection\t111\tBagging\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 5, trainingTestingFolds);
        System.out.println("FeatureSelection\t5\tBagging\t" + result);

        poolMethod = 1;
        prebuiltClassifiers = buildFeatureSelectionEnsembles(baseC, size, poolMethod, trainingTestingFolds);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 999, trainingTestingFolds);
        System.out.println("FeatureSelection\t999\tPartition\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 121354, trainingTestingFolds);
        System.out.println("FeatureSelection\tRandom\tPartition\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 111, trainingTestingFolds);
        System.out.println("FeatureSelection\t111\tPartition\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 5, trainingTestingFolds);
        System.out.println("FeatureSelection\t5\tPartition\t" + result);*/

        System.gc();
    }

    public static void crossValidateFeatureSelectionEnsembleMixed(String dataset) throws Exception {
        System.out.println("\nEvaluating Dataset - " + dataset);
        int numFolds = 10;

        Instances data;
        data = CrossValidation.openDataset(dataset);
        data.setClassIndex(data.numAttributes() - 1);

        double result;
        Instances[] trainingTestingFolds = CrossValidation.generateCrossValidationFolds(numFolds, data);
        Classifier[] prebuiltClassifiers;
        int poolMethod;
        int size = 50;

        poolMethod = 4; //BaggingMixed;          
        prebuiltClassifiers = buildFeatureSelectionEnsembles(null, size, poolMethod, trainingTestingFolds);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 999, trainingTestingFolds);
        System.out.println("FeatureSelection\t999\tMixed Bagging\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 121354, trainingTestingFolds);
        System.out.println("FeatureSelection\tRandom\tMixed Bagging\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 111, trainingTestingFolds);
        System.out.println("FeatureSelection\t111\tMixed Bagging\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 5, trainingTestingFolds);
        System.out.println("FeatureSelection\t5\tMixed Bagging\t" + result);

        poolMethod = 3; //RandomSubspacingMixed;
        prebuiltClassifiers = buildFeatureSelectionEnsembles(null, size, poolMethod, trainingTestingFolds);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 999, trainingTestingFolds);
        System.out.println("FeatureSelection\t999\tMixed Partition\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 121354, trainingTestingFolds);
        System.out.println("FeatureSelection\tRandom\tMixed Partition\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 111, trainingTestingFolds);
        System.out.println("FeatureSelection\t111\tMixed Partition\t" + result);

        result = CrossValidation.crossValidateFeatureSelectionEnsemble(prebuiltClassifiers, 5, trainingTestingFolds);
        System.out.println("FeatureSelection\t5\tMixed Partition\t" + result);

        System.gc();
    }

    /*public static void crossValidateFeatureSelectionEnsembleMixedPartition(String dataset) throws Exception {
        System.out.println("\nEvaluating Dataset - " + dataset);
        Instances data;
        Classifier classifier;
        double result;

        data = CrossValidation.openDataset(dataset);
        data.setClassIndex(data.numAttributes() - 1);

        //FeatureSelection + 999 + Bagging
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(3);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(999);
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(1);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t999\tMixed Partition\t" + result);

        //FeatureSelection + 111 + Bagging
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(3);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t111\tMixed Partition\t" + result);

        //FeatureSelection + 5 + Bagging
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(3);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(5);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t5\tMixed Partition\t" + result);
    }

    public static void crossValidateFeatureSelectionEnsembleMixedBagging(String dataset) throws Exception {
        System.out.println("\nEvaluating Dataset - " + dataset);
        Instances data;
        Classifier classifier;
        double result;

        data = CrossValidation.openDataset(dataset);
        data.setClassIndex(data.numAttributes() - 1);

        //FeatureSelection + 999 + Partition
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(4);
        //((FeatureSelectionEnsemble) classifier).setBaseC(baseC);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(999);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t999\tMixed Bagging\t" + result);

        //FeatureSelection + 111 + Partition
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(4);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t111\tMixed Bagging\t" + result);

        //FeatureSelection + 5 + Partition
        classifier = new FeatureSelectionEnsemble();
        ((FeatureSelectionEnsemble) classifier).setPoolMethod(4);
        ((FeatureSelectionEnsemble) classifier).setSelectionMethod(5);
        result = CrossValidation.crossValidate(10, classifier, data);
        System.out.println("FeatureSelection\t5\tMixed Bagging\t" + result);
    }*/


    private static void beam(originalharmonysearch.core.ValueRange[] parameterRanges, int cap, int maxIteration) throws Exception {
        originalharmonysearch.core.ValueRange[] beamValueRanges = new originalharmonysearch.core.ValueRange[4];
        beamValueRanges[0] = new originalharmonysearch.core.ValueRange(0.125, 5, true);
        beamValueRanges[1] = new originalharmonysearch.core.ValueRange(0.1, 10, true);
        beamValueRanges[2] = new originalharmonysearch.core.ValueRange(0.1, 10, true);
        beamValueRanges[3] = new originalharmonysearch.core.ValueRange(0.1, 5, true);
        BeamHarmonyMemory beamHarmonyMemory =
                new BeamHarmonyMemory(
                        parameterRanges,
                        beamValueRanges);
        iterativeTest(beamHarmonyMemory, cap, maxIteration);
    }

    /*private static void beam3(int cap, int maxIteration) throws Exception {
        ValueRange[] beamValueRanges = new ValueRange[4];
        beamValueRanges[0] = new ValueRange(0.125, 5, true);
        beamValueRanges[1] = new ValueRange(0.1, 10, true);
        beamValueRanges[2] = new ValueRange(0.1, 10, true);
        beamValueRanges[3] = new ValueRange(0.1, 5, true);
        BeamHarmonyMemory beamHarmonyMemory =
                new BeamHarmonyMemory(
                        originalharmonysearch.core.HarmonyMemory.createParameterRanges(20, 20, 0.8, 0.8, 0.85, 0.85, 0.001, 0.001),
                        beamValueRanges);
        iterativeTest(beamHarmonyMemory, cap, maxIteration);
    }

    private static void beam2(int cap, int maxIteration) throws Exception {
        ValueRange[] beamValueRanges = new ValueRange[4];
        beamValueRanges[0] = new ValueRange(0.125, 5, true);
        beamValueRanges[1] = new ValueRange(0.1, 10, true);
        beamValueRanges[2] = new ValueRange(0.1, 10, true);
        beamValueRanges[3] = new ValueRange(0.1, 5, true);
        BeamHarmonyMemory beamHarmonyMemory =
                new BeamHarmonyMemory(
                        originalharmonysearch.core.HarmonyMemory.createParameterRanges(20, 0.5),
                        beamValueRanges);
        iterativeTest(beamHarmonyMemory, cap, maxIteration);
    }*/

    private static void vessel(originalharmonysearch.core.ValueRange[] parameterRanges, int cap, int maxIteration) throws Exception {
        originalharmonysearch.core.ValueRange[] vesselValueRanges = new originalharmonysearch.core.ValueRange[4];
        vesselValueRanges[0] = new originalharmonysearch.core.ValueRange(0, 50, false);
        vesselValueRanges[1] = new originalharmonysearch.core.ValueRange(0, 50, false);
        vesselValueRanges[2] = new originalharmonysearch.core.ValueRange(0, 200, true);
        vesselValueRanges[3] = new originalharmonysearch.core.ValueRange(0, 200, true);
        VesselHarmonyMemory vesselHarmonyMemory = new VesselHarmonyMemory(parameterRanges, vesselValueRanges);
        iterativeTest(vesselHarmonyMemory, cap, maxIteration);
    }

    private static void constraintFour(originalharmonysearch.core.ValueRange[] parameterRanges, int cap, int maxIteration) throws Exception {
        originalharmonysearch.core.ValueRange[] constraintFourValueRanges = new originalharmonysearch.core.ValueRange[3];
        constraintFourValueRanges[0] = new originalharmonysearch.core.ValueRange(0, 10, true);
        constraintFourValueRanges[1] = new originalharmonysearch.core.ValueRange(0, 10, true);
        constraintFourValueRanges[2] = new originalharmonysearch.core.ValueRange(0, 10, true);

        ConstraintFourHarmonyMemory constraintFourHarmonyMemory = new ConstraintFourHarmonyMemory(parameterRanges, constraintFourValueRanges);

        iterativeTest(constraintFourHarmonyMemory, cap, maxIteration);
    }

    private static void spring(originalharmonysearch.core.ValueRange[] parameterRanges, int cap, int maxIteration) throws Exception {

        originalharmonysearch.core.ValueRange[] springValueRanges = new originalharmonysearch.core.ValueRange[3];
        springValueRanges[0] = new originalharmonysearch.core.ValueRange(0, 0.1, true);
        springValueRanges[1] = new originalharmonysearch.core.ValueRange(0, 0.5, true);
        springValueRanges[2] = new originalharmonysearch.core.ValueRange(8, 15, true);

        SpringHarmonyMemory springHarmonyMemory1 = new SpringHarmonyMemory(parameterRanges, springValueRanges);

        iterativeTest(springHarmonyMemory1, cap, maxIteration);
    }

    /*private static void iterativeResetableTest(HarmonyMemory harmonyMemory, int cap, int maxIteration) throws Exception {
            //int cap = 10;
            double[] capSpread = new double[cap];
            double[] capOriginal = new double[cap];
            double[] capInfluence = new double[cap];

            for (int i = 0; i < cap; i++) {
                //System.out.println("No Spread " + i);
                harmonyMemory.setSpread(false);
                harmonyMemory.reset();
                harmonyMemory.fill();
                harmonyMemory.iterate(maxIteration);
                //harmonyMemory.printHarmonies();
                //System.out.println();
                capOriginal[i] = harmonyMemory.best().getMerits()[0];
            }

            for (int i = 0; i < cap; i++) {
                //System.out.println("Spread " + i);
                harmonyMemory.setSpread(true);
                harmonyMemory.reset();
                harmonyMemory.fill();
                harmonyMemory.iterate(maxIteration);
                //harmonyMemory.printHarmonies();
                //System.out.println();
                capSpread[i] = harmonyMemory.best().getMerits()[0];
            }

            for (int i = 0; i < cap; i++) {
                //System.out.println("Spread " + i);
                harmonyMemory.setSpread(false);
                harmonyMemory.setInfluence(true);
                harmonyMemory.reset();
                harmonyMemory.fill();
                harmonyMemory.iterate(maxIteration);
                //harmonyMemory.printHarmonies();
                //System.out.println();
                capInfluence[i] = harmonyMemory.best().getMerits()[0];
            }

            int countSpread = 0;
            int countInfluence = 0;
            DescriptiveStatistics descriptiveStatisticsOriginal = new DescriptiveStatistics();
            DescriptiveStatistics descriptiveStatisticsSpread = new DescriptiveStatistics();
            DescriptiveStatistics descriptiveStatisticsInfluence = new DescriptiveStatistics();
            for (int i = 0; i < cap; i++) {
                //System.out.println(capSpread[i] + " " + capNoSpread[i]);

                descriptiveStatisticsOriginal.addValue(capOriginal[i]);
                descriptiveStatisticsSpread.addValue(capSpread[i]);
                descriptiveStatisticsInfluence.addValue(capInfluence[i]);

                if (capSpread[i] > capOriginal[i]) countSpread++;
                if (capInfluence[i] > capOriginal[i]) countInfluence++;
            }
            System.out.println("Count Spread " + countSpread);
            System.out.println("Count Influence " + countInfluence);
            System.out.println("SD\t" + descriptiveStatisticsOriginal.getStandardDeviation() + "\t" + descriptiveStatisticsSpread.getStandardDeviation() + "\t" + descriptiveStatisticsInfluence.getStandardDeviation());
            System.out.println("Max\t" + descriptiveStatisticsOriginal.getMax() + "\t" + descriptiveStatisticsSpread.getMax() + "\t" + descriptiveStatisticsInfluence.getMax());
            System.out.println("Mean\t" + descriptiveStatisticsOriginal.getMean() + "\t" + descriptiveStatisticsSpread.getMean() + "\t" + descriptiveStatisticsInfluence.getMean());

            TTestImpl tTest = new TTestImpl();
            System.out.println("T-Spread " + tTest.tTest(capSpread, capOriginal));
            System.out.println("T-Influence " + tTest.tTest(capInfluence, capOriginal));
        }*/


    private static void iterativeTest(originalharmonysearch.core.HarmonyMemory harmonyMemory, int cap, int maxIteration) throws Exception {
        //int cap = 10;
        double[] capSpread = new double[cap];
        double[] capOriginal = new double[cap];
        double[] capInfluence = new double[cap];


        for (int i = 0; i < cap; i++) {
            //System.out.println("Spread " + i);
            harmonyMemory.setSpread(true);
            harmonyMemory.reset();
            harmonyMemory.fill();
            harmonyMemory.iterate(maxIteration);
            //harmonyMemory.printHarmonies();
            //System.out.println();
            capSpread[i] = harmonyMemory.best().getMerits()[0];
        }

        for (int i = 0; i < cap; i++) {
            //System.out.println("No Spread " + i);
            harmonyMemory.setSpread(false);
            harmonyMemory.reset();
            harmonyMemory.fill();
            harmonyMemory.iterate(maxIteration);
            //harmonyMemory.printHarmonies();
            //System.out.println();
            capOriginal[i] = harmonyMemory.best().getMerits()[0];
        }

        for (int i = 0; i < cap; i++) {
            //System.out.println("Spread " + i);
            harmonyMemory.setSpread(false);
            harmonyMemory.setInfluence(true);
            harmonyMemory.reset();
            harmonyMemory.fill();
            harmonyMemory.iterate(maxIteration);
            //harmonyMemory.printHarmonies();
            //System.out.println();
            capInfluence[i] = harmonyMemory.best().getMerits()[0];
        }

        int countSpread = 0;
        int countInfluence = 0;
        DescriptiveStatistics descriptiveStatisticsOriginal = new DescriptiveStatistics();
        DescriptiveStatistics descriptiveStatisticsSpread = new DescriptiveStatistics();
        DescriptiveStatistics descriptiveStatisticsInfluence = new DescriptiveStatistics();
        for (int i = 0; i < cap; i++) {
            //System.out.println(capSpread[i] + " " + capNoSpread[i]);

            descriptiveStatisticsOriginal.addValue(capOriginal[i]);
            descriptiveStatisticsSpread.addValue(capSpread[i]);
            descriptiveStatisticsInfluence.addValue(capInfluence[i]);

            if (capSpread[i] > capOriginal[i]) countSpread++;
            if (capInfluence[i] > capOriginal[i]) countInfluence++;
        }
        System.out.println("Count Spread " + countSpread);
        System.out.println("Count Influence " + countInfluence);
        System.out.println("SD\t" + descriptiveStatisticsOriginal.getStandardDeviation() + "\t" + descriptiveStatisticsSpread.getStandardDeviation() + "\t" + descriptiveStatisticsInfluence.getStandardDeviation());
        System.out.println("Max\t" + descriptiveStatisticsOriginal.getMax() + "\t" + descriptiveStatisticsSpread.getMax() + "\t" + descriptiveStatisticsInfluence.getMax());
        System.out.println("Mean\t" + descriptiveStatisticsOriginal.getMean() + "\t" + descriptiveStatisticsSpread.getMean() + "\t" + descriptiveStatisticsInfluence.getMean());

        TTestImpl tTest = new TTestImpl();
        System.out.println("T-Spread " + tTest.tTest(capSpread, capOriginal));
        System.out.println("T-Influence " + tTest.tTest(capInfluence, capOriginal));
    }


    /*private void oldMain(String[] args) {
        //ResultFormatter.format();
        //new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0001, 0.01}, {5, 50}
        //{0, 0.1, 1}, {0, 0.5, 1}, {8, 15, 1}


        *//*ValueRange[] valueRanges = new ValueRange[numMusicians + 2];
        for (int i = 0; i < numMusicians; i++) {
            valueRanges[i] = new ValueRange(0, numNotes, false);
        }*//*
        //valueRanges[numMusicians] = new ValueRange(numHarmonies - 10, numHarmonies + 10, false);;
        //valueRanges[numMusicians + 1] = new ValueRange(0.5, 1, true);;
        //wrapper(30, 10, 0.85, 100, subsetEvaluator, data);


        //SpringHarmonyMemory harmonyMemory = new SpringHarmonyMemory(25, *//*3, *//*0.8);
        //statisticalReport();

        //SpringHarmonyMemory harmonyMemory = new SpringHarmonyMemory(25, 3, 0.8);
        *//*ConstraintFourHarmonyMemory harmonyMemory = new ConstraintFourHarmonyMemory(25, 3, 0.8);

        harmonyMemory.initialise();
        harmonyMemory.fill();
        harmonyMemory.iterate(50000);
        harmonyMemory.printHarmonies();*//*
        *//*System.out.println("Ours " + new SpringEvaluation().checkConstraints(new double[]{0.052310122724562355, 0.37183076590103425, 10.455416343702042}));
        System.out.println("Ours " + new SpringEvaluation().evaluate(new double[]{0.052310122724562355, 0.37183076590103425, 10.455416343702042}));
        //System.out.println(new BeamEvaluation().checkConstraints(new double[]{0.20573,3.47049,9.03662,0.20573}));
        //System.out.println("Arora " + new SpringEvaluation().checkConstraints(new double[]{0.053396,0.399180,9.185400}));
        System.out.println("IHS " + new SpringEvaluation().checkConstraints(new double[]{0.05115438,0.34987116,12.0764321}));
        System.out.println("IHS " + new SpringEvaluation().evaluate(new double[]{0.05115438,0.34987116,12.0764321}));
        //System.out.println("Bele " + new SpringEvaluation().checkConstraints(new double[]{0.050000,0.315900,14.25}));
        System.out.println("Coello " + new SpringEvaluation().checkConstraints(new double[]{0.051989,0.363965,10.890522}));
        System.out.println("Coello " + new SpringEvaluation().evaluate(new double[]{0.051989,0.363965,10.890522}));
*//*
        //System.out.println("Coello " + new BeamEvaluation().checkConstraints(new double[]{0.2088,3.4205,8.9975,0.2100}));
        //System.out.println("Coello " + new BeamEvaluation().evaluate(new double[]{0.2088, 3.4205, 8.9975, 0.2100}));

        //System.out.println("IHS " + new BeamEvaluation().checkConstraints(new double[]{0.20573, 3.47049, 9.03662, 0.20573}));
        //System.out.println("IHS " + new BeamEvaluation().evaluate(new double[]{0.20573, 3.47049, 9.03662, 0.20573}));

        //System.out.println(new UnconTwoEvaluation().evaluate(new double[]{0, -1}));
        //System.out.println(new ConstraintFiveEvaluation().evaluate(new double[]{2.2468258,2.381863}));
        //System.out.println(new ConstraintFourEvaluation().checkConstraints(new double[]{5,5,5}));
        //System.out.println(new ConstraintFourEvaluation().evaluate(new double[]{5,5,5}));
        //System.out.println(new UnconEvaluation().evaluate(new double[]{3.00000,3.99999}));


        //System.out.println("IHS " + new SpringEvaluation().checkConstraints(new double[]{0.05115438,0.34987116,12.0764321}));

        //statisticalReport();


        *//*HMS: 5 MS: 2 MI: 500000
        Parameters: {0.5,0.95}, {0.35,0.99}, {1.0E-5,0.01}
        Best Solution Found at : 457903
        (1)	-2.0224735637727037E-7	-1.0000013261429128	- 3.0000000000095626
        Number of Updates: 468 out of 500000*//*
        HarmonyMemoryOld lastResult = null;
        double temp;
        int count = 0;
        if (args[0].equals("spring")) {
            temp = 1;
            while (temp > 0.012681) {
                System.out.println(count);
                temp = springOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0001, 0.01}, {5, 50}}).getBestScore();
                count++;
            }
        } else if (args[0].equals("vessel")) {
            temp = 15000;
            while (temp > 7197.72895) {
                System.out.println(count);
                lastResult = vesselOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.00001, 0.01}, {5, 10}});
                temp = lastResult.getBestScore();
                count++;
            }
            lastResult.printSummary();
        } else if (args[0].equals("beam")) {
            temp = 5;
            while (temp > 2.1754) {
                System.out.println(count);
                //beamOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.00001, 0.01}, {5, 10}}).getBestScore();
                lastResult = beamOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.00001, 0.01}, {5, 10}});
                temp = lastResult.getBestScore();
                count++;
            }
            lastResult.printSummary();
            *//*HMS: 5 MS: 4 MI: 500000
            Parameters: {0.5,0.95}, {0.35,0.99}, {1.0E-5,0.01}
            Best Solution Found at : 424777
            (2)	0.21228524426298637	8.050951198000964	7.878075365900629	0.21229951804353825	- 2.175095482060236
            Number of Updates: 4074 out of 500000*//*
        } else if (args[0].equals("disjoint")) {
            temp = 20;
            while (temp > 13.590842061409647) {
                System.out.println(count);
                temp = disjointOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.00001, 0.01}, {25, 50}}).getBestScore();
                count++;
            }
        } else if (args[0].equals("constraintfive")) {
            temp = 20;
            while (temp > 13.590841776379554) {
                System.out.println(count);
                temp = constraintFiveOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.00001, 0.01}, {25, 50}}).getBestScore();
                count++;
            }
        } else if (args[0].equals("constraintfour")) {
            temp = 0;
            *//*int best = 0;
            for (int i=0; i<1000; i++) {
                System.out.println(count);
                double now = constraintFourOld(new double[][]{{0.5, 0.95}, {0.35, 0.999}, {0.0000001, 0.01}, {25, 50}});
                if (now > temp) {
                    best = i;
                    temp = now;
                }
                count++;
            }
            System.out.println(best);*//*
            while (temp < 0.9999999999999999) {
                System.out.println(count);
                temp = constraintFourOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0000000001, 0.01}, {25, 50}}).getBestScore();
                count++;
            }
        } else if (args[0].equals("uncon")) {
            temp = 2;
            while (temp > 1.000000000064621) {
                System.out.println(count);
                temp = unconOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.00000001, 0.01}, {25, 50}}).getBestScore();
                count++;
            }
        } else if (args[0].equals("uncontwo")) {
            temp = 4;
            while (temp > 3.000000000095626) {
                System.out.println(count);
                lastResult = unconTwoOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.000001, 0.01}, {25, 50}});
                temp = lastResult.getBestScore();
                count++;

                *//*HMS: 5 MS: 2 MI: 500000
                Parameters: {0.5,0.95}, {0.35,0.99}, {1.0E-6,0.01}
                Best Solution Found at : 321144
                (0)	-7.316132771698078E-8	-0.9999999798627552	- 3.0000000000018066
                Number of Updates: 397 out of 500000*//*

            }
            lastResult.printSummary();
        }


        //reconstruct harmony memory if downsized
        //springOld(new double[][] {{0.5,0.95},{0.35,0.99},{0.0005, 0.05}});
        //springOld(new double[][] {{0.75,0.95},{0.35,0.99},{0.0005, 0.05}});
        //springOld(new double[][] {{0.95,0.95},{0.35,0.99},{0.0005, 0.05}});
        *//*double temp = 5;
        int count = 0;
        while (temp >= 1.7248) {
            System.out.println(count);
            temp = beamOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.00001, 0.01}, {25, 50}});
            count++;
        }*//*
        *//*double temp = 15000;
        int count = 0;
        while (temp >= 7197.730) {
            System.out.println(count);
            temp = vesselOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.00001, 0.01}, {25, 50}});
            count++;
        }*//*

        //System.out.println(count);
        *//*springOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0001, 0.01}, {50, 50}});
        springOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0001, 0.01}, {40, 50}});
        springOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0001, 0.01}, {30, 50}});
        springOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0001, 0.01}, {20, 50}});
        springOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0001, 0.01}, {10, 50}});
        springOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0001, 0.01}, {5, 50}});*//*
        //closer harmony size better result?

        *//*springOld(new double[][]{{0.75, 0.95}, {0.35, 0.99}, {0.0005, 0.05}});
        springOld(new double[][]{{0.75, 0.95}, {0.35, 0.99}, {0.0005, 0.05}});
        springOld(new double[][]{{0.75, 0.95}, {0.35, 0.99}, {0.0005, 0.05}});

        springOld(new double[][]{{0.95, 0.95}, {0.35, 0.99}, {0.0005, 0.05}});
        springOld(new double[][]{{0.95, 0.95}, {0.35, 0.99}, {0.0005, 0.05}});
        springOld(new double[][]{{0.95, 0.95}, {0.35, 0.99}, {0.0005, 0.05}});*//*


        //springOld(new double[][] {{0.5,0.95},{0.55,0.99},{0.0005, 0.05}});
        //springOld(new double[][] {{0.5,0.95},{0.75,0.99},{0.0005, 0.05}});

        *//*unconOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.000001, 4}});
        unconOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.000001, 4}});
        unconOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.000001, 4}});*//*


        //springOld(new double[][] {{0.95,0.95},{0.35,0.99},{0.0005, 0.05}});
        //springOld(new double[][] {{0.95,0.95},{0.55,0.99},{0.0005, 0.05}});
        //springOld(new double[][] {{0.95,0.95},{0.75,0.99},{0.0005, 0.05}});

        //double[] temp;
        *//*int musicianSize = 3;
        int memorySize = 4;
        int maxIteration = 500000;
        //int lastBestIteration = 0;
        double[][] valueRanges = new double[][] {{0,0.1}, {0,0.5}, {0,20}};
        double[][] parameterRanges = new double[][] {{0.5,0.95},{0.35,0.99},{0.0005, 0.05}};
        //HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(musicianSize, memorySize, maxIteration, *//**//*0.95, 0.7, 0.05, *//**//*true, valueRanges, parameterRanges);
        HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(musicianSize, memorySize, maxIteration, true, valueRanges, parameterRanges);
        harmonyMemory.fill();
        harmonyMemory.iterate();*//*
        *//*for (int i=0; i<memorySize; i++) {
            temp = harmonyMemory.newHarmony();
            while (!SpringEvaluation.checkConstraints(temp)) {
                temp = harmonyMemory.newHarmony();
            }
            harmonyMemory.addHarmony(temp, i);
        }
        //harmonyMemory.printHorizontally();
        harmonyMemory.updateScores();*//*
        //harmonyMemory.printWorst();

        //System.out.println(harmonyMemory.newHarmony());

        //harmonyMemory.updateHarmonyMemory(new double[] {1,2,3,4,5,6,7,8,9,0});
        *//*for (int i=0; i<maxIteration; i++) {
            //System.out.println(i);
            temp = harmonyMemory.newHarmony();
            while (!SpringEvaluation.checkConstraints(temp)) {
                temp = harmonyMemory.newHarmony();
            }
            harmonyMemory.updateHarmonyMemory(temp);
            *//**//*if (harmonyMemory.updateHarmonyMemory(temp)) {
                //System.out.println(i);
                lastBestIteration = i;
            }*//**//*
        }*//*

        //harmonyMemory.printSummary();

        //harmonyMemory.printHorizontally();
        //harmonyMemory.updateScores();
        //harmonyMemory.printWorst();
        *//*harmonyMemory.printBest();
        System.out.println("Number of Updates: " + harmonyMemory.getUpdateCount() + " out of " + maxIteration);
        System.out.println("Last Best Iteration: " + lastBestIteration);*//*
    }*/

    private static double standardDeviation(double[] inputs) {
        double standardDeviation = 0d;
        double mean = mean(inputs);

        for (double input : inputs) {
            standardDeviation += Math.pow(input - mean, 2);
        }
        return Math.sqrt(standardDeviation / inputs.length);
    }

    private static double mean(double[] inputs) {
        double mean = 0d;
        for (double input : inputs) {
            mean += input;
        }
        return mean / inputs.length;
    }

/*    private static void statisticalReport() {
        HarmonyMemoryOld result;
        double avgUpdateCount;
        double lowerBestScore;
        double upperBestScore;
        double avgBestScore;
        double avgLastBestIteration;
        int numberOfRuns = 100;
        double[] results;

        NumberFormat formatter = new DecimalFormat("#.00");
        for (int j = 5; j <= 10; j++) {
            lowerBestScore = Double.MAX_VALUE;
            upperBestScore = 0d;
            avgUpdateCount = 0d;
            avgBestScore = 0d;
            avgLastBestIteration = 0d;
            results = new double[numberOfRuns];
            for (int i = 0; i < numberOfRuns; i++) {

                //MS
                //result = vesselOld(new double[][]{{0.85,0.85}, {0.8, 0.8}, {0.05, 0.05}, {(j-4) * 5, (j-4) * 10}});

                //HMCR
                result = vesselOld(new double[][]{{j / 10, 1}, {0.8, 0.8}, {0.05, 0.05}, {20, 20}});

                //result = vesselOld(new double[][]{{j/10, 1}, {0.8, 0.8}, {0.05, 0.05}, {20, 20}});
                //result = vesselOld(new double[][]{{0.85, 0.85}, {j/10, 1}, {0.01, 0.01}, {20, 20}});
                //result = vesselOld(new double[][]{{0.85, 0.85}, {0.9, 0.9}, {1/Math.pow(10, j-3), 1/Math.pow(10, j-4)}, {20, 20}});
                //result = vesselOld(new double[][]{{0.85, 0.85}, {0.8, 0.8}, {1 / Math.pow(10, j - 3), 0.01}, {20, 20}});
                avgUpdateCount += result.getUpdateCount();
                avgBestScore += result.getBestScore();
                results[i] = result.getBestScore();
                if (result.getBestScore() < lowerBestScore) {
                    lowerBestScore = result.getBestScore();
                }
                if (result.getBestScore() > upperBestScore) {
                    upperBestScore = result.getBestScore();
                }

                //System.out.print(result.getBestScore() + ",");

                avgLastBestIteration += result.getLastBestIteration();
            }
            //System.out.println();
            //MS
            //System.out.println((j-4) * 5 + "--" + (j-4) * 10 + " & " + formatter.format(avgBestScore/numberOfRuns)*//* + " & " + formatter.format(lowerBestScore)  + " & " +  formatter.format(upperBestScore)*//*  + " & " + formatter.format(standardDeviation(results))  + " & " + avgUpdateCount/numberOfRuns + " & " + avgLastBestIteration/numberOfRuns + " \\\\");

            //HMCR
            System.out.println((j + 0d) / 10 + "--" + 1 + " & " + formatter.format(avgBestScore / numberOfRuns)*//* + " & " + formatter.format(lowerBestScore)  + " & " +  formatter.format(upperBestScore)*//* + " & " + formatter.format(standardDeviation(results)) + " & " + avgUpdateCount / numberOfRuns + " & " + avgLastBestIteration / numberOfRuns + " \\\\");

            //System.out.println(1 / Math.pow(10, j - 3) + "--" + 0.01 + " & " + avgBestScore / numberOfRuns + " & " + avgUpdateCount / numberOfRuns + " & " + avgLastBestIteration / numberOfRuns + " //");
            lowerBestScore = Double.MAX_VALUE;
            upperBestScore = 0d;
            avgUpdateCount = 0d;
            avgBestScore = 0d;
            avgLastBestIteration = 0d;
            results = new double[numberOfRuns];
            for (int i = 0; i < numberOfRuns; i++) {

                //MS
                //result = vesselOld(new double[][]{{0.85,0.85}, {0.8, 0.8}, {0.05, 0.05}, {(j-4) * 10, (j-4) * 10}});

                //HMCR
                result = vesselOld(new double[][]{{j / 10, j / 10}, {0.8, 0.8}, {0.05, 0.05}, {20, 20}});

                //result = vesselOld(new double[][]{{0.85,0.85}, {j/10, j/10}, {0.01, 0.01}, {20, 20}});
                //result = vesselOld(new double[][]{{0.85, 0.85}, {0.8, 0.8}, {1 / Math.pow(10, j - 3), 1 / Math.pow(10, j - 3)}, {20, 20}});
                if (result.getBestScore() < lowerBestScore) {
                    lowerBestScore = result.getBestScore();
                }
                if (result.getBestScore() > upperBestScore) {
                    upperBestScore = result.getBestScore();
                }
                results[i] = result.getBestScore();
                avgUpdateCount += result.getUpdateCount();
                avgBestScore += result.getBestScore();
                avgLastBestIteration += result.getLastBestIteration();
            }

            //MS
            //System.out.println((j-4) * 10 + " & " + formatter.format(avgBestScore/numberOfRuns)*//* + " & " + formatter.format(lowerBestScore)  + " & " +  formatter.format(upperBestScore)*//*  + " & " +  formatter.format(standardDeviation(results))  + " & " + avgUpdateCount/numberOfRuns + " & " + avgLastBestIteration/numberOfRuns + " \\\\");

            //HMCR
            System.out.println((j + 0d) / 10 + " & " + formatter.format(avgBestScore / numberOfRuns)*//* + " & " + formatter.format(lowerBestScore)  + " & " +  formatter.format(upperBestScore)*//* + " & " + formatter.format(standardDeviation(results)) + " & " + avgUpdateCount / numberOfRuns + " & " + avgLastBestIteration / numberOfRuns + " \\\\");

            //System.out.println(1 / Math.pow(10, j - 3) + " & " + avgBestScore / numberOfRuns + " & " + avgUpdateCount / numberOfRuns + " & " + avgLastBestIteration / numberOfRuns + " //");
            System.out.println("\\hline");
        }
    }

    //springOld(new double[][]{{0.5, 0.95}, {0.35, 0.99}, {0.0001, 0.01}, {5, 50}})

    private static HarmonyMemoryOld springOld(double[][] parameterRanges) {
        HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(
                3, 50, 50000, true,
                new double[][]{{0, 0.1, 1}, {0, 0.5, 1}, {8, 15, 1}},
                parameterRanges, new SpringEvaluation());
        harmonyMemory.fill();
        harmonyMemory.iterate();
        //harmonyMemory.printSummary();
        return harmonyMemory;
    }

    private static HarmonyMemoryOld vesselOld(double[][] parameterRanges) {
        HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(
                4, 20, 2000, true,
                //new double[][] {{17,19,0}, {9,11,0}, {58,59,1}, {43,44,1}},
                new double[][]{{0, 50, 0}, {0, 50, 0}, {0, 200, 1}, {0, 200, 1}},
                parameterRanges, new VesselEvaluation());
        harmonyMemory.fill();
        harmonyMemory.iterate();
        //harmonyMemory.printSummary();
        return harmonyMemory;
        //System.out.println(18 * 0.0625 + " " + 10 * 0.0625);
        //System.out.println(0.6224 * 1.125 * 58.29015 * 43.69268 + 1.7781 * 0.625 * Math.pow(58.29015, 2) + 3.1661 * Math.pow(1.125, 2) * 43.69268 + 19.84 * Math.pow(1.125, 2) * 58.29015);
    }

    private static HarmonyMemoryOld beamOld(double[][] parameterRanges) {
        HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(
                4, 10, 500000, true,
                new double[][]{{0.125, 5, 1}, {0.1, 10, 1}, {0.1, 10, 1}, {0.1, 5, 1}},
                //new double[][] {{0.2,0.21,1}, {3.4,3.5,1}, {8.9,9.1,1}, {0.2,0.211,1}},
                parameterRanges, new BeamEvaluation());
        harmonyMemory.fill();
        harmonyMemory.iterate();
        //harmonyMemory.printSummary();
        return harmonyMemory;
        //System.out.println(18 * 0.0625 + " " + 10 * 0.0625);
        //System.out.println(0.6224 * 1.125 * 58.29015 * 43.69268 + 1.7781 * 0.625 * Math.pow(58.29015, 2) + 3.1661 * Math.pow(1.125, 2) * 43.69268 + 19.84 * Math.pow(1.125, 2) * 58.29015);
    }

    private static HarmonyMemoryOld disjointOld(double[][] parameterRanges) {
        HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(
                2, 50, 500000, true,
                new double[][]{{2, 3, 1}, {2, 3, 1}},
                parameterRanges, new DisjointEvaluation());
        harmonyMemory.fill();
        harmonyMemory.iterate();
        //harmonyMemory.printSummary();
        return harmonyMemory;
        //System.out.println(18 * 0.0625 + " " + 10 * 0.0625);
        //System.out.println(0.6224 * 1.125 * 58.29015 * 43.69268 + 1.7781 * 0.625 * Math.pow(58.29015, 2) + 3.1661 * Math.pow(1.125, 2) * 43.69268 + 19.84 * Math.pow(1.125, 2) * 58.29015);
    }

    private static HarmonyMemoryOld constraintFiveOld(double[][] parameterRanges) {
        HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(
                2, 50, 500000, true,
                new double[][]{{2, 3, 1}, {2, 3, 1}},
                parameterRanges, new ConstraintFiveEvaluation());
        harmonyMemory.fill();
        harmonyMemory.iterate();
        //harmonyMemory.printSummary();
        return harmonyMemory;
        //System.out.println(18 * 0.0625 + " " + 10 * 0.0625);
        //System.out.println(0.6224 * 1.125 * 58.29015 * 43.69268 + 1.7781 * 0.625 * Math.pow(58.29015, 2) + 3.1661 * Math.pow(1.125, 2) * 43.69268 + 19.84 * Math.pow(1.125, 2) * 58.29015);
    }

    private static HarmonyMemoryOld constraintFourOld(double[][] parameterRanges) {
        HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(
                3, 50, 200000, false,
                new double[][]{{0, 10, 1}, {0, 10, 1}, {0, 10, 1}},
                parameterRanges, new ConstraintFourEvaluation());
        harmonyMemory.fill();
        harmonyMemory.iterate();
        //harmonyMemory.printSummary();
        return harmonyMemory;
        //System.out.println(18 * 0.0625 + " " + 10 * 0.0625);
        //System.out.println(0.6224 * 1.125 * 58.29015 * 43.69268 + 1.7781 * 0.625 * Math.pow(58.29015, 2) + 3.1661 * Math.pow(1.125, 2) * 43.69268 + 19.84 * Math.pow(1.125, 2) * 58.29015);
    }

    private static HarmonyMemoryOld unconOld(double[][] parameterRanges) {
        HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(
                2, 50, 500000, true,
                new double[][]{{-50, 50, 1}, {-50, 50, 1}},
                parameterRanges, new UnconEvaluation());
        harmonyMemory.fill();
        harmonyMemory.iterate();
        //harmonyMemory.printSummary();
        return harmonyMemory;
        //System.out.println(18 * 0.0625 + " " + 10 * 0.0625);
        //System.out.println(0.6224 * 1.125 * 58.29015 * 43.69268 + 1.7781 * 0.625 * Math.pow(58.29015, 2) + 3.1661 * Math.pow(1.125, 2) * 43.69268 + 19.84 * Math.pow(1.125, 2) * 58.29015);
    }

    private static HarmonyMemoryOld unconTwoOld(double[][] parameterRanges) {
        HarmonyMemoryOld harmonyMemory = new HarmonyMemoryOld(
                2, 50, 500000, true,
                new double[][]{{-50, 50, 1}, {-50, 50, 1}},
                parameterRanges, new UnconTwoEvaluation());
        harmonyMemory.fill();
        harmonyMemory.iterate();
        //harmonyMemory.printSummary();
        return harmonyMemory;
        //System.out.println(18 * 0.0625 + " " + 10 * 0.0625);
        //System.out.println(0.6224 * 1.125 * 58.29015 * 43.69268 + 1.7781 * 0.625 * Math.pow(58.29015, 2) + 3.1661 * Math.pow(1.125, 2) * 43.69268 + 19.84 * Math.pow(1.125, 2) * 58.29015);
    }*/
}
