package hierarchicalinterpolation.control;

import fuzzy.FuzzyNumber;
import hierarchicalinterpolation.model.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;

/**
 * Created by IntelliJ IDEA.
 * User: rrd09
 * Date: 28/07/11
 * Time: 14:17
 * To change this template use File | Settings | File Templates.
 */
public class Process {

    private boolean verbose = true;

    public static RuleBase loadCrowdednessRuleBase() {
        RuleBase ruleBase = new RuleBase(new CrowdednessFunction());
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.0, 0.4, 1.4), new FuzzyNumber(0.0, 0.5, 1.5), new FuzzyNumber(0.0, 0.9, 1.9)}, new FuzzyNumber(0.0, 0.8, 1.8));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.1, 1.1, 2.1), new FuzzyNumber(0.0, 0.6, 1.6), new FuzzyNumber(3.1, 4.1, 5.1)}, new FuzzyNumber(0.0, 0.8, 1.8));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.2, 1.2, 2.2), new FuzzyNumber(0.0, 1.0, 2.0), new FuzzyNumber(7.3, 8.3, 9.3)}, new FuzzyNumber(0.0, 0.7, 1.7));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.0, 0.4, 1.4), new FuzzyNumber(3.0, 4.0, 5.0), new FuzzyNumber(0.0, 0.9, 1.9)}, new FuzzyNumber(0.8, 1.8, 2.8));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.0, 0.6, 1.6), new FuzzyNumber(3.6, 4.6, 5.6), new FuzzyNumber(3.2, 4.2, 5.2)}, new FuzzyNumber(0.5, 1.5, 2.5));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.0, 1.0, 2.0), new FuzzyNumber(3.8, 4.8, 5.8), new FuzzyNumber(7.1, 8.1, 9.1)}, new FuzzyNumber(0.4, 1.4, 2.4));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.0, 0.7, 1.7), new FuzzyNumber(7.6, 8.6, 9.6), new FuzzyNumber(0.0, 1.0, 2.0)}, new FuzzyNumber(1.8, 2.8, 3.8));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.1, 1.1, 2.1), new FuzzyNumber(7.1, 8.1, 9.1), new FuzzyNumber(3.7, 4.7, 5.7)}, new FuzzyNumber(1.3, 2.3, 3.3));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.0, 0.8, 1.8), new FuzzyNumber(7.1, 8.1, 9.1), new FuzzyNumber(6.9, 7.9, 8.9)}, new FuzzyNumber(0.7, 1.7, 2.7));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.2, 4.2, 5.2), new FuzzyNumber(0.0, 1.0, 2.0), new FuzzyNumber(0.0, 1.0, 2.0)}, new FuzzyNumber(1.1, 2.1, 3.1));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.4, 4.4, 5.4), new FuzzyNumber(0.0, 0.8, 1.8), new FuzzyNumber(3.9, 4.9, 5.9)}, new FuzzyNumber(0.5, 1.5, 2.5));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.5, 4.5, 5.5), new FuzzyNumber(0.0, 0.9, 1.9), new FuzzyNumber(7.3, 8.3, 9.3)}, new FuzzyNumber(0.3, 1.3, 2.3));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.2, 4.2, 5.2), new FuzzyNumber(3.7, 4.7, 5.7), new FuzzyNumber(0.0, 1.0, 2.0)}, new FuzzyNumber(3.2, 4.2, 5.2));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.4, 4.4, 5.4), new FuzzyNumber(3.2, 4.2, 5.2), new FuzzyNumber(3.9, 4.9, 5.9)}, new FuzzyNumber(2.0, 3.0, 4.0));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.7, 4.7, 5.7), new FuzzyNumber(4.0, 5.0, 6.0), new FuzzyNumber(7.2, 8.2, 9.2)}, new FuzzyNumber(1.7, 2.7, 3.7));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.1, 4.1, 5.1), new FuzzyNumber(7.4, 8.4, 9.4), new FuzzyNumber(0.0, 0.9, 1.9)}, new FuzzyNumber(4.5, 5.5, 6.5));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.2, 4.2, 5.2), new FuzzyNumber(7.5, 8.5, 9.5), new FuzzyNumber(3.9, 4.9, 5.9)}, new FuzzyNumber(3.1, 4.1, 5.1));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.2, 4.2, 5.2), new FuzzyNumber(7.7, 8.7, 9.7), new FuzzyNumber(6.8, 7.8, 8.8)}, new FuzzyNumber(2.5, 3.5, 4.5));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.6, 8.6, 9.6), new FuzzyNumber(0.2, 1.2, 2.2), new FuzzyNumber(0.0, 0.5, 1.5)}, new FuzzyNumber(2.3, 3.3, 4.3));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.4, 8.4, 9.4), new FuzzyNumber(0.1, 1.1, 2.1), new FuzzyNumber(3.0, 4.0, 5.0)}, new FuzzyNumber(1.4, 2.4, 3.4));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(6.8, 7.8, 8.8), new FuzzyNumber(0.0, 0.5, 1.5), new FuzzyNumber(7.4, 8.4, 9.4)}, new FuzzyNumber(0.5, 1.5, 2.5));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.1, 8.1, 9.1), new FuzzyNumber(3.9, 4.9, 5.9), new FuzzyNumber(0.0, 0.5, 1.5)}, new FuzzyNumber(5.0, 6.0, 7.0));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.2, 8.2, 9.2), new FuzzyNumber(3.4, 4.4, 5.4), new FuzzyNumber(3.6, 4.6, 5.6)}, new FuzzyNumber(3.2, 4.2, 5.2));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.4, 8.4, 9.4), new FuzzyNumber(3.3, 4.3, 5.3), new FuzzyNumber(6.8, 7.8, 8.8)}, new FuzzyNumber(2.5, 3.5, 4.5));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.3, 8.3, 9.3), new FuzzyNumber(7.2, 8.2, 9.2), new FuzzyNumber(0.0, 0.6, 1.6)}, new FuzzyNumber(6.7, 7.7, 8.7));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.4, 8.4, 9.4), new FuzzyNumber(6.8, 7.8, 8.8), new FuzzyNumber(3.4, 4.4, 5.4)}, new FuzzyNumber(4.7, 5.7, 6.7));
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.6, 8.6, 9.6), new FuzzyNumber(7.0, 8.0, 9.0), new FuzzyNumber(7.1, 8.1, 9.1)}, new FuzzyNumber(3.6, 4.6, 5.6));
        return ruleBase;
    }
    
    public static RuleBase loadExplosionRuleBase() {
        RuleBase ruleBase = new RuleBase(new ExplosionFunction());

        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.0,1.0,2.0), new FuzzyNumber(0.0,0.8,1.8)}, new FuzzyNumber(0.0,0.9,1.9)
);
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.0,1.0,2.0), new FuzzyNumber(3.4,4.4,5.4)}, new FuzzyNumber(0.0,0.7,1.7)
);
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(0.1,1.1,2.1), new FuzzyNumber(7.4,8.4,9.4)}, new FuzzyNumber(0.0,0.6,1.6)
);
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.5,4.5,5.5), new FuzzyNumber(0.0,0.5,1.5)}, new FuzzyNumber(3.2,4.2,5.2)
);
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(3.7,4.7,5.7), new FuzzyNumber(3.1,4.1,5.1)}, new FuzzyNumber(2.3,3.3,4.3)
);
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(4.2,5.2,6.2), new FuzzyNumber(6.9,7.9,8.9)}, new FuzzyNumber(1.9,2.9,3.9)
);
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.6,8.6,9.6), new FuzzyNumber(0.0,0.3,1.3)}, new FuzzyNumber(7.2,8.2,9.2)
);
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.0,8.0,9.0), new FuzzyNumber(3.5,4.5,5.5)}, new FuzzyNumber(4.5,5.5,6.5)
);
        ruleBase.add(new FuzzyNumber[]{
                new FuzzyNumber(7.7,8.7,9.7), new FuzzyNumber(7.6,8.6,9.6)}, new FuzzyNumber(3.7,4.7,5.7)
);

        return ruleBase;
    }    

    public static double getDelta(FuzzyNumber aStar, FuzzyNumber a1, FuzzyNumber a2) {
        return Math.abs((a1.getRep() - aStar.getRep()) / (a1.getRep() - a2.getRep()));
    }

    public static FuzzyNumber getIntermediateFuzzyTerm(FuzzyNumber a1, FuzzyNumber a2, double delta) {
        double newLeftShoulder = (1 - delta) * a1.getLeftShoulder() + delta * a2.getLeftShoulder();
        double newCentroid = (1 - delta) * a1.getCentroid() + delta * a2.getCentroid();
        double newRightShoulder = (1 - delta) * a1.getRightShoulder() + delta * a2.getRightShoulder();
        return new FuzzyNumber(newLeftShoulder, newCentroid, newRightShoulder);
    }

    public static double getScaleRate(FuzzyNumber aDash, FuzzyNumber aStar) {
        return (aStar.getRightShoulder() - aStar.getLeftShoulder()) /
                (aDash.getRightShoulder() - aDash.getLeftShoulder());
    }

    public static FuzzyNumber scale(FuzzyNumber fuzzyNumber, double scaleRate) throws Exception {
        if (scaleRate < 0) throw new Exception("Scale Rate (" + scaleRate + ") cannot be less than 0");
        double newLeftShoulder = (fuzzyNumber.getLeftShoulder() * (1 + 2 * scaleRate) +
                fuzzyNumber.getCentroid() * (1 - scaleRate) +
                fuzzyNumber.getRightShoulder() * (1 - scaleRate)) / 3;
        double newCentroid = (fuzzyNumber.getLeftShoulder() * (1 - scaleRate) +
                fuzzyNumber.getCentroid() * (1 + 2 * scaleRate) +
                fuzzyNumber.getRightShoulder() * (1 - scaleRate)) / 3;
        double newRightShoulder = (fuzzyNumber.getLeftShoulder() * (1 - scaleRate) +
                fuzzyNumber.getCentroid() * (1 - scaleRate) +
                fuzzyNumber.getRightShoulder() * (1 + 2 * scaleRate)) / 3;
        return new FuzzyNumber(newLeftShoulder, newCentroid, newRightShoulder);
    }

    public static double getMoveRatio(FuzzyNumber aDashScaled, FuzzyNumber aStar) throws Exception {
        double moveRatio;
        if (aStar.getLeftShoulder() >= aDashScaled.getLeftShoulder()) {
            moveRatio = (aStar.getLeftShoulder() - aDashScaled.getLeftShoulder()) * 3 /
                    (aDashScaled.getCentroid() - aDashScaled.getLeftShoulder());
            if ((moveRatio < 0) || (moveRatio > 1))
                throw new Exception("Move Ratio (" + moveRatio + ") cannot exceed [0,1]");
        } else {
            moveRatio = (aStar.getLeftShoulder() - aDashScaled.getLeftShoulder()) * 3 /
                    (aDashScaled.getRightShoulder() - aDashScaled.getCentroid());
            if ((moveRatio > 0) || (moveRatio < -1))
                throw new Exception("Move Ratio (" + moveRatio + ") cannot exceed [-1,0]");
        }
        return moveRatio;
    }

    public static FuzzyNumber move(FuzzyNumber fuzzyNumber, double moveRatio) {
        double moveRate = (moveRatio >= 0) ? moveRatio * (fuzzyNumber.getCentroid() - fuzzyNumber.getLeftShoulder()) / 3
                : moveRatio * (fuzzyNumber.getRightShoulder() - fuzzyNumber.getCentroid()) / 3;
        return new FuzzyNumber(fuzzyNumber.getLeftShoulder() + moveRate,
                fuzzyNumber.getCentroid() - 2 * moveRate,
                fuzzyNumber.getRightShoulder() + moveRate);
    }

    public static RuleBase randomDiscard(int minRuleCount, RuleBase ruleBase) {
        Random random = new Random();
        while (ruleBase.size() > minRuleCount) {
            ruleBase.remove(random.nextInt(ruleBase.size()));
        }
        return ruleBase;
    }

    public static Rule getIntermediateRule(Rule rule1, Rule rule2, Observation observation) {
        int dimensionToBeInterpolated = -1;
        int numberOfAntecedents = observation.getAntecedents().length;
        double deltaSum = 0;
        double delta;
        Rule intermediateRule = new Rule(rule1);

        for (int i = 0; i < numberOfAntecedents; i++) {
            if (observation.getAntecedents()[i] == null) dimensionToBeInterpolated = i;
        }

        if (dimensionToBeInterpolated >= 0) {
            //backward
            for (int i = 0; i < numberOfAntecedents; i++) {
                if (i != dimensionToBeInterpolated) {
                    delta = getDelta(observation.getAntecedents()[i],
                            rule1.getAntecedents()[i],
                            rule2.getAntecedents()[i]);
                    deltaSum = deltaSum + delta;
                    intermediateRule.getAntecedents()[i] =
                            getIntermediateFuzzyTerm(rule1.getAntecedents()[i], rule2.getAntecedents()[i], delta);
                }
            }
            delta = getDelta(observation.getConsequence(),
                    rule1.getConsequence(),
                    rule2.getConsequence());
            //    System.out.println("delta=" + delta);
            intermediateRule.setConsequence(
                    getIntermediateFuzzyTerm(rule1.getConsequence(), rule2.getConsequence(), delta));

            delta = delta * numberOfAntecedents - deltaSum;
            intermediateRule.getAntecedents()[dimensionToBeInterpolated] =
                    getIntermediateFuzzyTerm(
                            rule1.getAntecedents()[dimensionToBeInterpolated],
                            rule2.getAntecedents()[dimensionToBeInterpolated], delta);

        } else {
            //forward
            for (int i = 0; i < numberOfAntecedents; i++) {
                delta = getDelta(observation.getAntecedents()[i],
                        rule1.getAntecedents()[i],
                        rule2.getAntecedents()[i]);
                deltaSum = deltaSum + delta;
                intermediateRule.getAntecedents()[i] =
                        getIntermediateFuzzyTerm(rule1.getAntecedents()[i], rule2.getAntecedents()[i], delta);
            }
            delta = deltaSum / numberOfAntecedents;
            intermediateRule.setConsequence(
                    getIntermediateFuzzyTerm(rule1.getConsequence(), rule2.getConsequence(), delta));
        }

        return intermediateRule;
    }

    public static Rule transform(Rule intermediateRule, Observation observation) throws Exception {
        Rule transformedRule = new Rule(intermediateRule);
        int dimensionToBeInterpolated = -1;
        int numberOfAntecedents = observation.getAntecedents().length;
        double sSum = 0;
        double s;
        double mSum = 0;
        double m;

        for (int i = 0; i < numberOfAntecedents; i++) {
            if (observation.getAntecedents()[i] == null) dimensionToBeInterpolated = i;
        }

        if (dimensionToBeInterpolated >= 0) {
            //backward
            for (int i = 0; i < numberOfAntecedents; i++) {
                if (i != dimensionToBeInterpolated) {
                    s = getScaleRate(intermediateRule.getAntecedents()[i], observation.getAntecedents()[i]);
                    intermediateRule.getAntecedents()[i] = scale(intermediateRule.getAntecedents()[i], s);
                    sSum = sSum + s;

                    m = getMoveRatio(intermediateRule.getAntecedents()[i], observation.getAntecedents()[i]);
                    transformedRule.getAntecedents()[i] = move(intermediateRule.getAntecedents()[i], m);
                    mSum = mSum + m;
                }
            }

            s = getScaleRate(intermediateRule.getConsequence(), observation.getConsequence());
            intermediateRule.setConsequence(scale(intermediateRule.getConsequence(), s));
            m = getMoveRatio(intermediateRule.getConsequence(), observation.getConsequence());
            transformedRule.setConsequence(move(intermediateRule.getConsequence(), m));

            s = s * numberOfAntecedents - sSum;
            m = m * numberOfAntecedents - mSum;
            transformedRule.getAntecedents()[dimensionToBeInterpolated] =
                    move(scale(intermediateRule.getAntecedents()[dimensionToBeInterpolated], s), m);
        } else {
            //forward
            for (int i = 0; i < numberOfAntecedents; i++) {
                s = getScaleRate(intermediateRule.getAntecedents()[i], observation.getAntecedents()[i]);
                intermediateRule.getAntecedents()[i] = scale(intermediateRule.getAntecedents()[i], s);
                sSum = sSum + s;

                m = getMoveRatio(intermediateRule.getAntecedents()[i], observation.getAntecedents()[i]);
                transformedRule.getAntecedents()[i] = move(intermediateRule.getAntecedents()[i], m);
                mSum = mSum + m;
            }
            s = sSum / numberOfAntecedents;
            m = mSum / numberOfAntecedents;
            transformedRule.setConsequence(move(scale(intermediateRule.getConsequence(), s), m));
        }
        return transformedRule;
    }

    public static Rule[] getClosestRules(RuleBase ruleBase,
                                         Observation observation,
                                         int numberOfClosestRules,
                                         boolean useWeightedConsequenceDistance) {
        sortRuleBase(ruleBase, observation, useWeightedConsequenceDistance);
        Rule[] closestRules = new Rule[numberOfClosestRules];
        closestRules[0] = ruleBase.get(0).clone();
        int currentRuleIndex = 1;
        int startIndex = 1;
        boolean valid = true;
        while (currentRuleIndex != numberOfClosestRules) {
            for (int i = startIndex; i < ruleBase.size(); i++) {
                Rule currentRule = ruleBase.get(i);
                valid = true;
                for (int j = 0; j < currentRule.getAntecedents().length; j++) {
                    if (observation.getAntecedents()[j] == null) {
                        valid = valid & (currentRule.getConsequence()
                                .distanceTo(observation.getConsequence()) *
                                closestRules[currentRuleIndex - 1].getConsequence()
                                        .distanceTo(observation.getConsequence()) < 0);
                    } else {
                        valid = valid &
                                (currentRule.getAntecedents()[j]
                                        .distanceTo(observation.getAntecedents()[j]) *
                                        closestRules[currentRuleIndex - 1].getAntecedents()[j]
                                                .distanceTo(observation.getAntecedents()[j]) < 0);
                    }
                }
                if (valid == true) {
                    closestRules[currentRuleIndex] = ruleBase.get(i).clone();
                    startIndex = i;
                    currentRuleIndex++;
                    i = ruleBase.size();
                }
                //startIndex = i;

            }
            //currentRuleIndex ++;
        }
        return closestRules;
    }

    public static void sortRuleBase(RuleBase ruleBase, Observation observation, boolean useWeightedConsequenceDistance) {
        Collections.sort(ruleBase, new RuleComparator(observation, useWeightedConsequenceDistance));
    }

    public static void printRules(Rule[] rules) {
        for (int i = 0; i < rules.length; i++) {
            for (FuzzyNumber fuzzyNumber : rules[i].getAntecedents()) {
                System.out.print(fuzzyNumber.toString() + "\t");
            }
            System.out.println(rules[i].getConsequence().toString());
        }
    }

    public static void printRule(Rule rule) {

        for (FuzzyNumber fuzzyNumber : rule.getAntecedents()) {
            if (fuzzyNumber != null) {
                System.out.print(fuzzyNumber.toString() + "\t");
            } else {
                System.out.print("(null, null, null)" + "\t");
            }
        }
        if (rule.getConsequence() != null) {
            System.out.println(rule.getConsequence().toString());
        } else {
            System.out.println("(null, null, null)");
        }
    }


    public static void printRules(RuleBase ruleBase) {
        for (int i = 0; i < ruleBase.size(); i++) {
            for (FuzzyNumber fuzzyNumber : ruleBase.get(i).getAntecedents()) {
                System.out.print(fuzzyNumber.toString() + "\t");
            }
            System.out.println(ruleBase.get(i).getConsequence().toString());
        }
        System.out.println("----------------------------");
    }

    public static void printRulesInt(RuleBase ruleBase) {
        for (int i = 0; i < ruleBase.size(); i++) {
            for (FuzzyNumber fuzzyNumber : ruleBase.get(i).getAntecedents()) {
                System.out.print(fuzzyNumber.toStringInt() + "\t");
            }
            System.out.println(ruleBase.get(i).getConsequence().toStringInt());
        }
    }

    public static void printRulesTex(RuleBase ruleBase) {
        for (int i = 0; i < ruleBase.size(); i++) {
            System.out.print("C" + (i+1) + " & ");
            for (FuzzyNumber fuzzyNumber : ruleBase.get(i).getAntecedents()) {
                System.out.print(fuzzyNumber.toString() + " & ");
            }
            System.out.println(ruleBase.get(i).getConsequence().toString() + "\\\\\\hline");
        }
        System.out.println("----------------------------");
    }
}
