package hierarchicalinterpolation.control;

import fuzzy.FuzzyNumber;
import fuzzy.TrapezoidalFuzzyNumber;
import hierarchicalinterpolation.model.*;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Random;

/**
 * Created by IntelliJ IDEA.
 * User: rrd09
 * Date: 28/07/11
 * Time: 20:11
 * To change this template use File | Settings | File Templates.
 */
public class RuleBaseGenerator {

    private float[] minAntecedentRanges;
    private float[] maxAntecedentRanges;
    private float minConsequenceRange;// = Float.MAX_VALUE;
    private float maxConsequenceRange;// = Float.MIN_VALUE;

    private void containRule(ArrayList<Rule> rules, Rule rule) {
        for (int i = 0; i < rules.size(); i++) {
            if (rules.get(i).getAntecedents()[0].equals(rule.getAntecedents()[0]) &
                    rules.get(i).getAntecedents()[1].equals(rule.getAntecedents()[1]) &
                    rules.get(i).getAntecedents()[2].equals(rule.getAntecedents()[2])) {
                System.out.println("CNM");
            }
            /*boolean identical = false;

            for (int j = 0; j < rules.get(i).getAntecedents().length; j++) {
                identical = identical | rules.get(i).getAntecedents()[j].equals(rule.getAntecedents()[j]);
                if () identical = identical |
            }

            for (FuzzyNumber fuzzyNumber : rules.get(i).getAntecedents()) {
                if (fuzzyNumber.equals(rule.getAntecedents()[i]))
            }*/
        }
    }

    public int[] getNextIndices(int[] currentIndices, float[] ranges) {
        int[] nextIndices = new int[ranges.length];

        if (currentIndices == null) {
            for (int i = 0; i < nextIndices.length; i++) {
                nextIndices[i] = 0;
            }
            return nextIndices;
        }

        boolean increment = false;
        for (int i = nextIndices.length - 1; i >= 0; i--) {
            if (i == nextIndices.length - 1) {
                if (currentIndices[i] == ranges[i]) {
                    increment = true;
                    nextIndices[i] = 0;
                } else {
                    nextIndices[i] = currentIndices[i] + 1;
                }
            } else {
                if (increment == true) {
                    if (currentIndices[i] == ranges[i]) {
                        if (i == 0) return null;
                        increment = true;
                        nextIndices[i] = 0;
                    } else {
                        increment = false;
                        nextIndices[i] = currentIndices[i] + 1;
                    }
                } else {
                    nextIndices[i] = currentIndices[i];
                }
            }

        }
        return nextIndices;
    }

    public Observation newObservation(int numberOfVariables, float[] ranges) {
        FuzzyNumber[] observationAntecedents = new FuzzyNumber[numberOfVariables];
        Random random = new Random();
        float a1 = 1 + random.nextInt((int) ranges[0]);
        float a2 = 1 + random.nextInt((int) ranges[1]);
        float a3 = 1 + random.nextInt((int) ranges[2]);
        float a4 = 1 + random.nextInt((int) ranges[3]);
        observationAntecedents[0] = new TrapezoidalFuzzyNumber(a1 - 1.1, a1 - 0.5, a1 + 0.5, a1 + 1.1);
        observationAntecedents[1] = new TrapezoidalFuzzyNumber(a2 - 1.1, a2 - 0.5, a2 + 0.5, a2 + 1.1);
        observationAntecedents[2] = new TrapezoidalFuzzyNumber(a3 - 1.1, a3 - 0.5, a3 + 0.5, a3 + 1.1);
        observationAntecedents[3] = new TrapezoidalFuzzyNumber(a4 - 1.1, a4 - 0.5, a4 + 0.5, a4 + 1.1);

        Observation observation = new Observation(observationAntecedents);
        return observation;
    }

    public float[] getCrispInputs(int numberOfVariables, Observation observation) {
        float[] crispInput = new float[numberOfVariables];
        for (int i = 0; i < crispInput.length; i++) {
            crispInput[i] = observation.getAntecedents()[i].getRepresentativeValue();
        }
        return crispInput;
    }

    private void assignValueRanges(float[][] antecedentCentroids) {

        minAntecedentRanges = new float[antecedentCentroids.length];
        maxAntecedentRanges = new float[antecedentCentroids.length];

        for (int i = 0; i < antecedentCentroids.length; i++) {
            minAntecedentRanges[i] = Float.MAX_VALUE;
            maxAntecedentRanges[i] = Float.MIN_VALUE;
            for (int j = 0; j < antecedentCentroids[i].length; j++) {
                if (antecedentCentroids[i][j] < minAntecedentRanges[i]) {
                    //System.out.println(antecedentCentroids[i][j]);
                    minAntecedentRanges[i] = antecedentCentroids[i][j];
                }
                if (antecedentCentroids[i][j] > maxAntecedentRanges[i])
                    maxAntecedentRanges[i] = antecedentCentroids[i][j];
            }
            minAntecedentRanges[i] = roundToOneDecimal(minAntecedentRanges[i] - 1f);
            maxAntecedentRanges[i] = roundToOneDecimal(maxAntecedentRanges[i] + 1f);
        }

        /*for (int i = 0; i < antecedentCentroids.length; i++) {
            System.out.println(i + " range = " + minAntecedentRanges[i] + " - " + maxAntecedentRanges[i]);
        }*/

    }

    public RuleBase generateRuleBase(TestFunction testFunction, int numberOfFuzzySetsPerVariable, int numberOfPoints) {
        int numberOfRules = (int) (Math.pow(numberOfFuzzySetsPerVariable, testFunction.getNumberOfVariables()));
        RuleBase ruleBase = new RuleBase(testFunction.getMinAntecedentRanges(),
                testFunction.getMaxAntecedentRanges(),
                testFunction.getMinConsequenceRange(),
                testFunction.getMaxConsequenceRange());
        //ArrayList<Rule> rules = new ArrayList<Rule>(numberOfRules);
        //float[][] crispAntecedents = new float[testFunction.getNumberOfVariables()][numberOfFuzzySetsPerVariable];
        //float[] crispConsequence = new float[numberOfRules];
        ArrayList<CrispRule> crispRuleArrayList = new ArrayList<CrispRule>(numberOfRules);
        Random random = new Random();
        for (int i = 0; i < numberOfRules; i++) {
            CrispRule crispRule = new CrispRule(testFunction.getNumberOfVariables());
            for (int j = 0; j < testFunction.getNumberOfVariables(); j++) {
                float min = testFunction.getMinAntecedentRanges()[j];
                float max = testFunction.getMaxAntecedentRanges()[j];
                crispRule.setAntecedent(j, min + random.nextFloat() * (max - min));
            }
            crispRule.setConsequence(testFunction.calculate(crispRule.getAntecedents()));
            crispRuleArrayList.add(crispRule);
        }
        for (int i = 0; i < numberOfRules; i++) {
            ruleBase.add(crispRuleArrayList.get(i).fuzzify(numberOfPoints, random, numberOfFuzzySetsPerVariable, ruleBase.newRule()));
        }
        return ruleBase;
    }

    public ArrayList<Rule> generateRules(TestFunction testFunction, int numberOfFuzzySetsPerVariable) {
        int numberOfRules = (int) (Math.pow(numberOfFuzzySetsPerVariable, testFunction.getNumberOfVariables()));
        ArrayList<Rule> rules = new ArrayList<Rule>();
        float[][] antecedentCentroids = new float[testFunction.getNumberOfVariables()][numberOfFuzzySetsPerVariable];
        float[] consequenceCentroids = new float[numberOfRules];
        Random random = new Random();

        for (int i = 0; i < testFunction.getNumberOfVariables(); i++) {
            float min = testFunction.getMinAntecedentRanges()[i];
            float max = testFunction.getMaxAntecedentRanges()[i];
            for (int j = 0; j < numberOfFuzzySetsPerVariable; j++) {
                float spread = (max - min) / numberOfFuzzySetsPerVariable;
                antecedentCentroids[i][j] = roundToOneDecimal(min + spread * j + random.nextFloat() * spread);
            }
        }

        assignValueRanges(antecedentCentroids);

        float[] ranges = new float[testFunction.getNumberOfVariables()];
        for (int i = 0; i < testFunction.getNumberOfVariables(); i++) {
            ranges[i] = numberOfFuzzySetsPerVariable - 1;
        }
        int[] currentIndices = getNextIndices(null, ranges);

        //float[] consequences = new float[numberOfRules];

        while (rules.size() < numberOfRules) {
            float[] input = new float[testFunction.getNumberOfVariables()];
            FuzzyNumber[] fuzzyInput = new FuzzyNumber[testFunction.getNumberOfVariables()];

            for (int i = 0; i < testFunction.getNumberOfVariables(); i++) {
                fuzzyInput[i] = new TrapezoidalFuzzyNumber(
                        roundToOneDecimal(antecedentCentroids[i][currentIndices[i]] - random.nextFloat() * 1f),
                        roundToOneDecimal(antecedentCentroids[i][currentIndices[i]] - random.nextFloat() * 0.5f),
                        roundToOneDecimal(antecedentCentroids[i][currentIndices[i]] + random.nextFloat() * 0.5f),
                        roundToOneDecimal(antecedentCentroids[i][currentIndices[i]] + random.nextFloat() * 1f));
                input[i] = fuzzyInput[i].getRepresentativeValue();
            }

            consequenceCentroids[rules.size()] = roundToOneDecimal(testFunction.calculate(input));
            FuzzyNumber fuzzyResult = new TrapezoidalFuzzyNumber(
                    consequenceCentroids[rules.size()] - 1d,
                    consequenceCentroids[rules.size()] - 0.5d,
                    consequenceCentroids[rules.size()] + 0.5d,
                    consequenceCentroids[rules.size()] + 1d);
            Rule rule = new Rule(fuzzyInput, fuzzyResult,
                    maxAntecedentRanges,
                    minAntecedentRanges,
                    testFunction.getMaxConsequenceRange(),
                    testFunction.getMinConsequenceRange());
            rules.add(rule);

            currentIndices = getNextIndices(currentIndices, ranges);
        }

        for (int i = 0; i < numberOfRules; i++) {
            if (consequenceCentroids[i] < minConsequenceRange) minConsequenceRange = consequenceCentroids[i];
            if (consequenceCentroids[i] > maxConsequenceRange) maxConsequenceRange = consequenceCentroids[i];
        }

        for (int i = 0; i < numberOfRules; i++) {
            rules.get(i).setMaxConsequenceRange(maxConsequenceRange + 1f);
            rules.get(i).setMinConsequenceRange(minConsequenceRange - 1f);
        }

        return rules;
    }

    /*public ArrayList<Rule> generate(TestFunction testFunction, int numberOfInput) {
        ArrayList<Rule> rules = new ArrayList<Rule>();
        float[][] repValues = new float[testFunction.getNumberOfVariables()][numberOfInput];
        float[][] leftValues = new float[testFunction.getNumberOfVariables()][numberOfInput];
        float[][] rightValues = new float[testFunction.getNumberOfVariables()][numberOfInput];
        float[] result = new float[40];
        float[] leftResult = new float[40];
        float[] rightResult = new float[40];
        Random random = new Random();
        for (int i = 0; i < testFunction.getNumberOfVariables(); i++) {
            float min = testFunction.getMinAntecedentRanges()[i];
            float max = testFunction.getMaxAntecedentRanges()[i];
            for (int j = 0; j < numberOfInput; j++) {
                int spread = (int) Math.floor((max - min) / numberOfInput);
                repValues[i][j] = min + spread * j + random.nextInt(spread);

            }
        }

        for (int j = 0; j < 38; j++) {
            float[] input = new float[testFunction.getNumberOfVariables()];
            FuzzyNumber[] fuzzyInput = new FuzzyNumber[testFunction.getNumberOfVariables()];
            for (int i = 0; i < testFunction.getNumberOfVariables(); i++) {
                input[i] = repValues[i][random.nextInt(numberOfInput)];
                fuzzyInput[i] = new  TrapezoidalFuzzyNumber(input[i] - 1d, input[i] - 0.5d, input[i] + 0.5d, input[i] + 1d);
            }
            result[j] = roundToOneDecimal(testFunction.calculate(input));
            leftResult[j] = result[j] - 1 - random.nextInt(2);
            rightResult[j] = 2 * result[j] - leftResult[j];
            FuzzyNumber fuzzyResult = new  TrapezoidalFuzzyNumber(leftResult[j], result[j]-0.5d, result[j]+0.5d, rightResult[j]);
            Rule rule = new Rule(fuzzyInput, fuzzyResult, testFunction.getMaxAntecedentRanges(),
                    testFunction.getMinAntecedentRanges(), testFunction.getMaxConsequenceRange(),
                    testFunction.getMinConsequenceRange());
            if (!rules.contains(rule)) {
                containRule(rules, rule);
                rules.add(rule);
            } else {
                j = j - 1;                                 s
            }
            *//* (FuzzyNumber[] antecedents, FuzzyNumber consequence,
        float[] maxAntecedentRanges, float[] minAntecedentRanges,
        float maxConsequenceRange, float minConsequenceRange) {*//*
        }

        float[] input38 = new float[testFunction.getNumberOfVariables()];
        FuzzyNumber[] fuzzyInput = new FuzzyNumber[testFunction.getNumberOfVariables()];
        for (int i = 0; i < testFunction.getNumberOfVariables(); i++) {
            input38[i] = repValues[i][0];
            fuzzyInput[i] = new  TrapezoidalFuzzyNumber(input38[i] - 1d, input38[i]-0.5d, input38[i]+0.5d, input38[i] + 1d);
        }
        result[38] = roundToOneDecimal(testFunction.calculate(input38));
        leftResult[38] = result[38] - 1d;
        rightResult[38] = 2 * result[38] - leftResult[38];
        FuzzyNumber fuzzyResult = new  TrapezoidalFuzzyNumber(leftResult[38], result[38]-0.5d, result[38]+0.5d, rightResult[38]);
        Rule rule38 = new Rule(fuzzyInput, fuzzyResult, testFunction.getMaxAntecedentRanges(),
                testFunction.getMinAntecedentRanges(), testFunction.getMaxConsequenceRange(),
                testFunction.getMinConsequenceRange());
        rules.add(rule38);

        float[] input39 = new float[testFunction.getNumberOfVariables()];
        FuzzyNumber[] fuzzyInput39 = new FuzzyNumber[testFunction.getNumberOfVariables()];
        for (int i = 0; i < testFunction.getNumberOfVariables(); i++) {
            input39[i] = repValues[i][numberOfInput - 1];
            fuzzyInput39[i] = new  TrapezoidalFuzzyNumber(input39[i] - 1d, input39[i]-0.5d, input39[i]+0.5d, input39[i] + 1d);
        }
        result[39] = roundToOneDecimal(testFunction.calculate(input39));
        leftResult[39] = result[39] - 1d;
        rightResult[39] = 2 * result[39] - leftResult[39];
        FuzzyNumber fuzzyResult39 = new  TrapezoidalFuzzyNumber(leftResult[39], result[39]-0.5d, result[39]+0.5d, rightResult[39]);
        Rule rule39 = new Rule(fuzzyInput39, fuzzyResult39, testFunction.getMaxAntecedentRanges(),
                testFunction.getMinAntecedentRanges(), testFunction.getMaxConsequenceRange(),
                testFunction.getMinConsequenceRange());
        rules.add(rule39);


        return rules;

    }
*/
    private float roundToOneDecimal(float d) {
        DecimalFormat twoDForm = new DecimalFormat("#.#");
        return Float.valueOf(twoDForm.format(d));
    }

}
