package huang;

public class Point {
  /** a point consists of the x and y values */
  double x, y;

  public Point(double x, double y) {
    this.x = x;
    this.y = y;
  }

  public double getX() {
    return x;
  }

  public double getY() {
    return y;
  }

  /** the addition of two points */
  public Point add(Point d) {
    double px;
    Point p;
    if (y != d.getY()) {
      throw new RuntimeException("two points should have the same y values");
    }
    px = x + d.x;
    p = new Point(px, y);
    return p;
  }

  /** multiple a point with a double value */
  public Point multiply(double weight) {
    double px;
    Point p;
    px = x * weight;
    p = new Point(px, y);
    return p;
  }

  /** calculate the interpolated point by two poins and lambda*/
  public Point calInterpolation(Point endP, double lambda) {
    Point p = (this.multiply(1 - lambda)).add(endP.multiply(lambda));
    return p;
  }

  /** test if the point is positive */
  public boolean isPositive() {
    if ( (x >= 0) && (y >= 0)) {
      return true;
    }
    else {
      return false;
    }
  }

  /** test if two points are equal */
  public boolean isEqual(Point a) {
    double XDif = getX() - a.getX();
    double YDif = getY() - a.getY();
    if ( (java.lang.Math.abs(XDif) > 1e-3) || (java.lang.Math.abs(YDif) > 1e-3)) {
      return false;
    }
    else {
      return true;
    }
  }

  /** calculate the lambda of the point between two other points */
  public double calLambda(Point startP, Point endP) {
    if ( (y != startP.getY()) || (y != endP.getY())) {
      throw new RuntimeException("y value should be the same");
    }

    if ( (x < startP.getX()) || (x > endP.getX())) {
      throw new RuntimeException("x value should be between those" +
                                " of the two interpotion points");
    }
    double dis1 = this.distance(startP);
    double dis2 = startP.distance(endP);
    return dis1 / dis2;
  }

  /** work out the distance between two points */
  public double distance(Point p) {
    return java.lang.Math.sqrt( (x - p.x) * (x - p.x) + (y - p.y) * (y - p.y));
  }

  /** work out the artificial point by two points and the specified
   * ratio of their y distances to the unkown point
   */
  public Point calArtificialPoint(Point rightP, double YRatio) {
    double XVal = (1-YRatio)*(getX()) + YRatio*(rightP.getX());
    double YVal = (1-YRatio)*(getY()) + YRatio*(rightP.getY());
    Point result = new Point(XVal, YVal);
    return result;
  }

  public String toString() {
    //return " Point(" + x + "," + y + ")";
    return new PrintfFormat("Point (%3.2f").sprintf(x) +
        new PrintfFormat(", %3.2f)").sprintf(y);
  }

  public static void main(String[] args) {
    Point p1 = new Point(2.0, 3);
    Point p2 = new Point(3, 3);
    Point sum = p1.add(p2);
    Point p3 = p1.multiply(2);
    System.out.println("p1: " + p1);
    System.out.println("p2: " + p2);
    System.out.println("sum: " + sum);
    System.out.println("p3: " + p3);
  }

}
