package huang;

import huang.Point;

/** represent a polygonal fuzzy set */
public class PolyN {
  private int numPoints;
  private Point[] points;
  private boolean[] isLevelArtificiallyAdded;
  private boolean isOddPoints;
  public final int CEILH;
  public final int FLOORH;
  private final int LEFT = 0;
  private final int RIGHT = 1;
  /************************ options ***************************/
  /** WeightREPType specifies the REP used
   * 0: average REP;
   * 1: the obsolete compatible REP;
   * 2: the new compatible REP definition which simply assigns
   * artificial odd points as 0 //not finalized
   * 30: the compatible REP definition for triangular with 5 odd points
   * 31: the compatible REP definition for hexagonal with 10 odd points
   * 4: average weighted REP;
   * 5: center of the core REP
   */
  public int WeightREPType = 5;
  // the number of artificial alpha cuts added to the polygonal fuzzy set
  public int numArtificialLevels = 0;



  public PolyN(Point[] a, boolean isAddArtificial) {
    int n = a.length;
    if (n % 2 == 0) {
      isOddPoints = false;
    }
    else {
      isOddPoints = true;
    }
    if(isAddArtificial) {
      numPoints = n + 2 * numArtificialLevels * (ceilHalf(n) - 1);
    } else {
      numPoints = n;
    }

    CEILH = ceilHalf(numPoints);
    FLOORH = floorHalf(numPoints);

    points = new Point[numPoints];
    isLevelArtificiallyAdded = new boolean[ceilHalf(numPoints)];

    if(isAddArtificial) {
      int pointIdx = 0;
      //assigne all points including the artificial odd points
      for (int i = 0; i < n; i++) {
        points[pointIdx] = a[i];
        if(pointIdx < ceilHalf(numPoints)) {
          isLevelArtificiallyAdded[pointIdx] = false;
        }
        pointIdx++;
        if ( (i == ceilHalf(n) - 1) && (!isOddPoints)) {
          continue;
        }
        if (i == n - 1) {
          break;
        }
        Point left = a[i];
        Point right = a[i + 1];
        for (int j = 0; j < numArtificialLevels; j++) {
          double YRatio = (double) (j + 1) / (double) (numArtificialLevels + 1);
          points[pointIdx] = left.calArtificialPoint(right, YRatio);
          if(pointIdx < ceilHalf(numPoints)) { //tag the original non-fake levels
            isLevelArtificiallyAdded[pointIdx] = true;
          }
          pointIdx++;
        }
      }
//      for (int i = 0; i < numPoints; i++) {
//        System.out.println("points[" + i + "] = " + points[i]);
//      }
    }else {
      for (int i = 0; i < n; i++) {
        points[i] = a[i];
      }
    }
  }

  public int getNumPoints() {
    return numPoints;
  }

  public Point getPoint(int index) {
    if ( (index < 0) || (index >= numPoints)) {
      throw new RuntimeException("illegal index for point");
    }
    else {
      return points[index];
    }
  }

  public Point[] getPoints() {
    return points;
  }

  //get the i-th support length (i={0, 1, ..., ceilHalf(numPoints)-1}), the last one
  //might be zero
  public double support(int index) {
    if ( (index < 0) || (index > ceilHalf(numPoints) - 1)) {
      throw new RuntimeException("illegal index for support");
    }

    Point p1, p2;
    p1 = getPoint(index);
    p2 = getPoint(numPoints - 1 - index);
//    if (java.lang.Math.abs(p1.getY()-p2.getY()) < 1e-4) {
//      throw new RuntimeException("y values of " + p1 + " and " + p2 +
//                                 " should be the same");
//    }
    return p2.getX() - p1.getX();
  }

  //get the i-th left slope length (i={0, 1, ..., ceilHalf(numPoints)-2})
  public double leftSlope(int index) {
    if ( (index < 0) || (index > ceilHalf(numPoints) - 2)) {
      throw new RuntimeException("illegal index for left slope");
    }

    Point p1, p2;
    p1 = getPoint(index);
    p2 = getPoint(index + 1);
    return p2.getX() - p1.getX();
  }

  //get the i-th right slope length (i={0, 1, ..., ceilHalf(numPoints)-2})
  public double rightSlope(int index) {
    if ( (index < 0) || (index > ceilHalf(numPoints) - 2)) {
      throw new RuntimeException("illegal index for right slope");
    }

    Point p1, p2;
    p1 = getPoint(numPoints - 1 - index);
    p2 = getPoint(numPoints - 2 - index);
    return p1.getX() - p2.getX();
  }

  /** upgraded scale ratios (or called scale criterias) from 0 to
   *  numPoints-1, the (CEILH-1)-th and the (numPoints-CEILH)-th
   *  (if isOddPoints == false) are not used
   *  Note that these are calculated from top to bottom
   */
  public double[] calSLinear(PolyN a) {
    double[] scaleCriterias = new double[numPoints];
    for(int i = CEILH-2; i >=0; i--) {
      if(leftSlope(i) != 0) {
        scaleCriterias[i] = a.leftSlope(i) / leftSlope(i);
      } else {
        //scaleCriterias[i] = -1; //don't know how to handle this
        scaleCriterias[i] = a.leftSlope(i) / (getPoint(i+1).getY()
                                              - getPoint(i).getY());
      }
      if(rightSlope(i) != 0) {
        scaleCriterias[numPoints - 1 - i] = a.rightSlope(i) / rightSlope(i);
      } else {
        //scaleCriterias[numPoints - 1 - i] = -1; //don't know how to handle this
        scaleCriterias[numPoints - 1 - i]
            = a.rightSlope(i) / (getPoint(i+1).getY()- getPoint(i).getY());
      }
    }

    System.out.println("the scale criterias are ");
    for(int i = 0; i < numPoints; i++) {
      System.out.println("scale criteria " + i + " is "+ scaleCriterias[i]);
      if(0 - scaleCriterias[i]  > 1e-3) {
        throw new RuntimeException("scale criterias should be positive");
      }
    }
    return scaleCriterias;
  }


//scale rates range from 0 to ceilHalf(numPoints)-1
  public double[] calsLinear(double scaleRateTop, double[] scaleCriterias) {
     if (scaleCriterias.length != numPoints) {
       throw new RuntimeException("the length of scaleCriterias is not correct");
     }
     double[] scaleRates = new double[FLOORH];
     if(isOddPoints == true) {
       scaleRates[FLOORH-1] =  scaleCriterias[FLOORH-1]*leftSlope(FLOORH-1)
           + scaleCriterias[numPoints-FLOORH]*rightSlope(FLOORH-1);
       scaleRates[FLOORH-1] /= support(FLOORH-1);
     } else {
       scaleRates[FLOORH-1] = scaleRateTop;
     }
     for (int i = FLOORH-2; i >= 0; i--) {
       scaleRates[i] = scaleRates[i+1]*support(i+1)
           + scaleCriterias[i]*leftSlope(i)
           + scaleCriterias[numPoints-1-i]*rightSlope(i);
       scaleRates[i] /= support(i);
     }
     return scaleRates;
   }


// public double[] calSLinear(PolyN a) {
//    double[] scaleCriterias = new double[numPoints];
//    for(int i = 1; i <= CEILH-1; i++) {
//      if((i==CEILH-1)&&(isOddPoints)) {
//        scaleCriterias[i] = 0;
//      }
//      if(leftSlope(i-1) != 0) {
//        scaleCriterias[i] = a.leftSlope(i - 1) / leftSlope(i - 1);
//      } else {
//        scaleCriterias[i] = -1; // don't know how to handle this
//      }
//      if(rightSlope(i-1) != 0) {
//        scaleCriterias[numPoints - 1 - i] = a.rightSlope(i-1)
//            / rightSlope(i-1);
//      } else {
//        scaleCriterias[numPoints - 1 - i] = -1; // don't know how to handle this
//      }
//    }
//
//    System.out.println("the scale criterias are ");
//    for(int i = 0; i < numPoints; i++) {
//      System.out.println("scale criteria " + i + " is "+ scaleCriterias[i]);
//      if(scaleCriterias[i] < 0) {
//        throw new RuntimeException("scale criterias should be positive");
//      }
//    }
//    return scaleCriterias;
//  }

  //scale rates range from 0 to floorHalf(numPoints)-1
//  public double[] calsLinear(double scaleRateBot, double[] scaleCriterias) {
//    if (scaleCriterias.length != numPoints) {
//      throw new RuntimeException("the length of scaleCriterias is not correct");
//    }
//    double[] scaleRates = new double[FLOORH];
//    scaleRates[0] = scaleRateBot;
//
//    for (int i = 1; i < FLOORH; i++) {
//      scaleRates[i] = scaleRates[i-1]*support(i-1)
//          - scaleCriterias[i]*leftSlope(i-1)
//          - scaleCriterias[numPoints-1-i]*rightSlope(i-1);
//      scaleRates[i] /= support(i);
//    }
//    return scaleRates;
//  }

  //scale ratio from 1 to floorHalf(numPoints)-1, the 0-th is not used
  public double[] calS(PolyN a) {
    double[] scaleRatios = new double[FLOORH];
    double val, vala;
    for (int i = 1; i < FLOORH; i++) {
      val = this.support(i) / this.support(i - 1);
      vala = a.support(i) / a.support(i - 1);
      if (vala >= val) {
        scaleRatios[i] = (vala - val) / (1 - val);
      }
      else {
        scaleRatios[i] = (vala - val) / val;
      }
    }

    for (int i = 1; i < FLOORH; i++) {
      if ( (scaleRatios[i] < -1) || (scaleRatios[i] > 1)) {
        throw new RuntimeException("invalid scale ratio");
      }
    }
    return scaleRatios;
  }

  //scale rates range from 0 to floorHalf(numPoints)-1
  public double[] cals(double scaleRateBot, double[] scaleRatios) {
    if (scaleRatios.length != FLOORH) {
      throw new RuntimeException("the length of scaleRatios is not correct");
    }
    double[] scaleRates = new double[FLOORH];
    scaleRates[0] = scaleRateBot;
    double val;

    for (int i = 1; i < FLOORH; i++) {
      if ( (scaleRatios[i] >= 0) && (scaleRatios[i] <= 1)) {
        val = support(i) / support(i - 1);
        scaleRates[i] = (scaleRatios[i] * (1 - val) / val + 1) * scaleRates[i -
            1];
      }
      else {
        scaleRates[i] = (scaleRatios[i] + 1) * scaleRates[i - 1];
      }
    }
    return scaleRates;
  }

  /** scale one fuzzy set uing specified scale rates */
  public PolyN scale(double[] scaleRates) {
    if (scaleRates.length != FLOORH) {
      throw new RuntimeException("the size of scaleRates is not correct!");
    }

    for (int i = 0; i < FLOORH; i++) {
      if (scaleRates[i] < 0) {
        throw new RuntimeException("scale rates should not be negative");
      }
    }

    PolyN newPolyN;
    double newXa0;
    PolyN result;
    double totalWeights = 0;
    double[] r = new double[CEILH - 1];
    double[] d = new double[numPoints];
    double[] dis = new double[numPoints];
    Point[] points = new Point[numPoints];
    double[] weights = calWeights(WeightREPType);

    d[0] = 0;
    if (isOddPoints) {
      d[numPoints - CEILH] = 0;
    }
    else {
      d[numPoints - CEILH] = support(CEILH - 1) * scaleRates[CEILH - 1];
    }
    for (int i = 0; i < CEILH - 1; i++) {
      System.out.println("leftSlope[" + i + "] =  " + leftSlope(i));
      System.out.println("rightSlope[" + i + "] =  " + rightSlope(i));
      r[i] = leftSlope(i) / rightSlope(i);
      System.out.println("r[" + i + "] =  " + r[i]);
      if (support(i + 1) == 0) {
        d[i + 1] = r[i] * support(i) * scaleRates[i] / (1 + r[i]);
        d[numPoints - 1 - i] = support(i) * scaleRates[i] / (1 + r[i]);
      }
      else {
        d[i +
            1] = r[i] *
            (support(i) * scaleRates[i] - support(i + 1) * scaleRates[i + 1]) /
            (1 + r[i]);
        d[numPoints - 1 -
            i] = (support(i) * scaleRates[i] - support(i + 1) * scaleRates[i +
                  1]) / (1 + r[i]);
      }
      System.out.println("d[" + (i + 1) + "] =  " + d[i + 1]);

      System.out.println("d[" + (numPoints - 1 - i) + "] =  " + d[numPoints - 1 - i]);
    }
    for (int i = 0; i < numPoints; i++) {
      for (int j = 0; j <= i; j++) {
        dis[i] = dis[i] + d[j];
      }
      System.out.println("dis[" + i + "] =  " + dis[i]);
    }

    newXa0 = calREP();
    for (int i = 0; i < numPoints; i++) {
      newXa0 = newXa0 - weights[i] * dis[i];
      totalWeights = totalWeights + weights[i];
    }
    newXa0 = newXa0 / totalWeights;

    for (int i = 0; i < numPoints; i++) {
      points[i] = new Point(newXa0 + dis[i], getPoint(i).getY());
    }

    result = new PolyN(points, false);
    if (!result.isConvex()) {
      throw new RuntimeException("non-convex fuzzy set result ");
    }
    if (!result.isPaired()) {
      throw new RuntimeException("non-paired fuzzy set result ");
    }
    return result;
  }

  /** calculate the new move ratios (or called move criterias)
   * they are the ratios between the moved-left-slope/the-right-slope
   * and those of the orignial fuzzy set. they are ranged from 0
   * to CEILH-2 */
  public double[] calMLinear(PolyN a) {
    double[] moveCriterias = new double[CEILH-1];
    double val, vala;
    for (int i = 0; i < CEILH - 1; i++) {
      if((this.leftSlope(i)!=0)&(this.rightSlope(i)!=0)) {
        val = this.leftSlope(i) / this.rightSlope(i);
      } else if(this.leftSlope(i) == 0) {
        val = (getPoint(i+1).getY() - getPoint(i).getY()) / this.rightSlope(i);
      } else if (this.rightSlope(i) == 0) {
        val = this.leftSlope(i) / (getPoint(i+1).getY() - getPoint(i).getY());
      } else {
        throw new RuntimeException("the rectangular input should not "
                                   + "exist in the move transformation");
      }

      if(val <= 0) {
        throw new RuntimeException("val should be positive after the treatment");
      }

      if (a.rightSlope(i) != 0) {
        vala = a.leftSlope(i) / a.rightSlope(i);
      } else {
        //throw new RuntimeException("a small modification is needed here");
        // indicate the moved fuzzy set should also has vertical right slope
        moveCriterias[i] = -1d;
        continue;
      }

      moveCriterias[i] = vala / val;
      if(java.lang.Math.abs(moveCriterias[i]) < 1e-3) {
        moveCriterias[i] = 0;
      }
    }

    for (int i = 0; i < CEILH - 1; i++) {
      if ((moveCriterias[i] < 0)&&(moveCriterias[i] != -1)) {
        throw new RuntimeException("move criterias should be positive "
                                    + " except value -1");
      }
    }
    return moveCriterias;
  }

  /** move fuzzy set with specified move criterias -- the ratios of the
   * ratios between left slopes and right slopes before and after the
   * move transformation, indices are ranged from 0 to CEILH-2 */
  public PolyN moveLinear(double[] moveCriterias) {
    if (moveCriterias.length != CEILH-1) {
      throw new RuntimeException("the size of move criterias is not correct!");
    }

    for (int i = 0; i < CEILH-1; i++) {
      if (moveCriterias[i] < 0) {
        throw new RuntimeException("move criterias should not be negative");
      }
    }

    PolyN newPolyN;
    double newXa0;
    PolyN result;
    double totalWeights = 0;
    double[] r = new double[CEILH - 1];
    double[] d = new double[numPoints];//
    double[] dis = new double[numPoints];
    Point[] points = new Point[numPoints];
    double[] weights = calWeights(WeightREPType);

    d[0] = 0;
    if (isOddPoints) {
      d[numPoints - CEILH] = 0;
    }
    else {
      d[numPoints - CEILH] = support(CEILH - 1);
    }
    for (int i = 0; i < CEILH - 1; i++) {
      System.out.println("leftSlope[" + i + "] =  " + leftSlope(i));
      System.out.println("rightSlope[" + i + "] =  " + rightSlope(i));
      //r[i] = leftSlope(i) / rightSlope(i);
      if(moveCriterias[i] == -1d) { //the right slope means to be vertical
        r[i] = -1d;
      } else {
        r[i] = moveCriterias[i] * leftSlope(i) / rightSlope(i);
      }
      System.out.println("r[" + i + "] =  " + r[i]);
      if (support(i + 1) == 0) {
        if(r[i] != -1d) {
          d[i + 1] = r[i] * support(i) / (1 + r[i]);
          d[numPoints - 1 - i] = support(i) / (1 + r[i]);
        } else {
          d[i + 1] = support(i);
          d[numPoints - 1 - i] = 0d;
        }
      }
      else {
        if(r[i] != -1d) {
          d[i + 1] = r[i] * (support(i) - support(i + 1))
              / (1 + r[i]);
          d[numPoints - 1 - i] = (support(i) - support(i + 1)) / (1 + r[i]);
        } else {
          d[i + 1] = support(i)- support(i + 1);
          d[numPoints - 1 - i] = 0d;
        }
      }
      System.out.println("d[" + (i + 1) + "] =  " + d[i + 1]);

      System.out.println("d[" + (numPoints - 1 - i) + "] =  " + d[numPoints - 1 - i]);
    }
    for (int i = 0; i < numPoints; i++) {
      for (int j = 0; j <= i; j++) {
        dis[i] = dis[i] + d[j];
      }
      System.out.println("dis[" + i + "] =  " + dis[i]);
    }

    newXa0 = calREP();
    for (int i = 0; i < numPoints; i++) {
      newXa0 = newXa0 - weights[i] * dis[i];
      totalWeights = totalWeights + weights[i];
    }
    newXa0 = newXa0 / totalWeights;

    for (int i = 0; i < numPoints; i++) {
      points[i] = new Point(newXa0 + dis[i], getPoint(i).getY());
    }

    result = new PolyN(points, false);
    if (!result.isConvex()) {
      throw new RuntimeException("non-convex fuzzy set result ");
    }
    if (!result.isPaired()) {
      throw new RuntimeException("non-paired fuzzy set result ");
    }
    return result;
  }



  /** calculate the move ratio of specified level from one fuzzy
   *  set to the other
   */
  public double calM(PolyN p, int level) {
    if ( (level < 0) || (level > CEILH - 2)) {
      throw new RuntimeException("wrong level to calculate move ratio");
    }

    double REPDif = this.calREP() - p.calREP();
    if (java.lang.Math.abs(REPDif) > 1e-3) {
      throw new RuntimeException("two Polygonal sets should have the same REP");
    }

    if (!isEqualUnder(p, level)) {
      throw new RuntimeException("two polygonals should be equal under level" +
                                 level);
    }

    //System.out.println("calM()");
    double[] weights = calWeights(WeightREPType);
    double val = 0;
    double X = this.getPoint(level).getX();
    double newX = p.getPoint(level).getX();
    double dif = newX - X;
    System.out.println("dif = " + dif);
    double extX;
    double moveRatio;
    if (dif >= 0) {
      extX = calExtPoint(level, RIGHT);
      System.out.println("extX - X = " + extX + "-" + X);
      if (extX < X) {
        throw new RuntimeException("extreme move distance should be positive");
      }
      moveRatio = dif / (extX - X);
      if (java.lang.Math.abs(moveRatio - 1d) < 1e-3) {
        moveRatio = 1d;
      }
      return moveRatio;
    }
    else {
      extX = calExtPoint(level, LEFT);
      if (extX >= X) {
        throw new RuntimeException("extreme move distance should be positive");
      }
      moveRatio = dif / (X - extX);
      if (java.lang.Math.abs(moveRatio + 1d) < 1e-3) {
        moveRatio = -1d;
      }
      return moveRatio;
    }
  }

  /** calculate the move ratios from one fuzzy set to the other
   */
  public double[] calM(PolyN p) throws CloneNotSupportedException {
    double[] moveRatios = new double[CEILH - 1];
    PolyN me = (PolyN)this.clone();

    for (int i = 0; i < CEILH - 1; i++) {
      moveRatios[i] = me.calM(p, i);
      me = me.move(moveRatios[i], i);
    }
    if (! (me.isEqualUnder(p, CEILH))) {
      throw new RuntimeException(
          "the move ratio calculation is not consistent with the move");
    }
    return moveRatios;
  }

  /** return the extreme move position not causing the
   * upper non-convixity for the specified level and
   * move direction
   */
  public double calExtPointUp(int level, int dirFlag) {
    double[] weights = calWeights(WeightREPType);
    double subTotalWeights;
    double totalWeights = 0;
    double totalWeightsX = 0;
    double val = 0;
    double valTest = 0;
    double extX1;
    double pointsDis;

    for (int i = level; i < CEILH; i++) {
      totalWeights = totalWeights + weights[i];
      totalWeightsX = totalWeightsX + weights[i] * getPoint(i).getX();
    }

    if (isOddPoints) {
      totalWeights = totalWeights - weights[CEILH - 1] / 2d;
      totalWeightsX = totalWeightsX - (weights[CEILH -
                                       1] / 2d) * getPoint(CEILH - 1).getX();
    }

    //	System.out.println("totalWeights = " + totalWeights);
    //System.out.println("totalWeightsX = " + totalWeightsX);

    if (totalWeights == 0) {
      throw new RuntimeException("the total weights should be non-zero");
    }
    else {
      if ( ( (totalWeights > 0) && (dirFlag == RIGHT))
          || ( (totalWeights < 0) && (dirFlag == LEFT))) {
        for (int k = level + 1; k < CEILH; k++) {
          subTotalWeights = 0;
          //either weight is negative or zeor but the closest up negative
          if ( (weights[k] < 0) ||
              ( (weights[k] == 0) && (upClosestNonZeroWeight(k) < 0))) {
            for (int m = k; m < CEILH; m++) {
              subTotalWeights = subTotalWeights + weights[m];
            }
            if (isOddPoints) {
              subTotalWeights = subTotalWeights - weights[CEILH - 1] / 2d;
            }
            val = val + (support(k - 1) - support(k)) * subTotalWeights;
            System.out.println("val = " + val);
          }
        }
        // begin test.
        for (int k = level + 1; k < CEILH; k++) {
          subTotalWeights = 0;
          if ( (weights[k] > 0) ||
              ( (weights[k] == 0) && (upClosestNonZeroWeight(k) > 0))) {
            for (int m = k; m < CEILH; m++) {
              subTotalWeights = subTotalWeights + weights[m];
            }
            if (isOddPoints) {
              subTotalWeights = subTotalWeights - weights[CEILH - 1] / 2d;
            }
            valTest = valTest + (support(k - 1) - support(k)) * subTotalWeights;
          }
        }
        if ( ( (totalWeightsX - val) / totalWeights) <
            ( (totalWeightsX - valTest) / totalWeights)) {
          throw new RuntimeException(
              "the left extrem poin should less than the right extrm point!");
        }
        // end test.
      }
      else {
        for (int k = level + 1; k < CEILH; k++) {
          subTotalWeights = 0;
          if ( (weights[k] > 0) ||
              ( (weights[k] == 0) && (upClosestNonZeroWeight(k) > 0))) {
            for (int m = k; m < CEILH; m++) {
              subTotalWeights = subTotalWeights + weights[m];
            }
            if (isOddPoints) {
              subTotalWeights = subTotalWeights - weights[CEILH - 1] / 2d;
            }
            val = val + (support(k - 1) - support(k)) * subTotalWeights;
          }
        }
        // begin test.
        for (int k = level + 1; k < CEILH; k++) {
          subTotalWeights = 0;
          if ( (weights[k] < 0) ||
              ( (weights[k] == 0) && (upClosestNonZeroWeight(k) < 0))) {
            for (int m = k; m < CEILH; m++) {
              subTotalWeights = subTotalWeights + weights[m];
            }
            if (isOddPoints) {
              subTotalWeights = subTotalWeights - weights[CEILH - 1] / 2d;
            }
            valTest = valTest + (support(k - 1) - support(k)) * subTotalWeights;
          }
        }
        if ( ( (totalWeightsX - val) / totalWeights) >
            ( (totalWeightsX - valTest) / totalWeights)) {
          throw new RuntimeException(
              "the left extrem poin should less than the right extrm point!");
        }
        // end test.
      }
      extX1 = (totalWeightsX - val) / totalWeights;
      return extX1;
    }
  }

  /** return the extreme move position not causing both the
   * upper and below non-convixities for the specified level and
   * move direction
   */
  public double calExtPoint(int level, int dirFlag) {
    //System.out.println("calExtPoint()");

    double extX1, extX2, extX;
    extX1 = calExtPointUp(level, dirFlag);
    if (level == 0) {
      System.out.println("calEXtPoint: constrained by the upper part");
      return extX1;
    }

    if (dirFlag == RIGHT) {
      extX2 = getPoint(level - 1).getX() + support(level - 1) - support(level);
      if (extX1 < extX2) {
        extX = extX1;
        System.out.println("calEXtPoint: constrained by the upper part");
      }
      else {
        extX = extX2;
        System.out.println("calEXtPoint: constrained by the lower part");
      }
    }
    else {
      extX2 = getPoint(level - 1).getX();
      if (extX1 > extX2) {
        extX = extX1;
        System.out.println("calEXtPoint: constrained by the upper part");
      }
      else {
        extX = extX2;
        System.out.println("calEXtPoint: constrained by the lower part");
      }
    }
    return extX;
  }

  /** return the extreme move positions not causing the
   * upper non-convixities for the specified level and
   * move direction
   */
  public double[] calExtPointsUp(int level, int dirFlag) {
    double[] weights = calWeights(WeightREPType);
    double totalWeights = 0;
    double extXs[] = new double[numPoints - 2 * level];
    int index;

    for (int i = level; i < CEILH; i++) {
      totalWeights = totalWeights + weights[i];
    }

    if (totalWeights == 0) {
      return null;
    }
    extXs[0] = calExtPointUp(level, dirFlag);

    if ( ( (totalWeights > 0) && (dirFlag == RIGHT))
        || ( (totalWeights < 0) && (dirFlag == LEFT))) {
      for (int k = level + 1; k < CEILH; k++) {
        if ( (weights[k] > 0) ||
            ( (weights[k] == 0) && (upClosestNonZeroWeight(k) > 0))) {
          extXs[k - level] = extXs[k - level - 1];
        }
        else {
          extXs[k - level] = extXs[k - level - 1] + support(k - 1) - support(k);
        }
      }
    }
    else {
      for (int k = level + 1; k < CEILH; k++) {
        if ( (weights[k] < 0) ||
            ( (weights[k] == 0) && (upClosestNonZeroWeight(k) < 0))) {
          extXs[k - level] = extXs[k - level - 1];
        }
        else {
          extXs[k - level] = extXs[k - level - 1] + support(k - 1) - support(k);
        }
      }
    }

    for (int k = numPoints - CEILH; k <= numPoints - 1 - level; k++) {
      extXs[k - level] = extXs[numPoints - 1 - level - k] + support(numPoints - 1 - k);
    }
    return extXs;
  }

  /** move the fuzzy set in the specified level with move ratio */
  public PolyN move(double moveRatio, int level) {
    if ( (level < 0) || (level > CEILH - 2)) {
      throw new RuntimeException("wrong level move");
    }

    if ( (moveRatio > 1d) || (moveRatio < -1d)) {
      throw new RuntimeException("move ratio is not valid");
    }

    Point[] points = new Point[numPoints];
    PolyN result;
    double X, extX, extXUp, appMoveRatio;
    double[] newX = new double[numPoints];
    double[] extXs = new double[numPoints - 2 * level];
    if (moveRatio >= 0) {
      X = getPoint(level).getX();
      extX = calExtPoint(level, RIGHT);
      extXUp = calExtPointUp(level, RIGHT);
      extXs = calExtPointsUp(level, RIGHT);
      appMoveRatio = moveRatio * (extX - X) / (extXUp - X);
      for (int i = 0; i < numPoints; i++) {
        newX[i] = getPoint(i).getX();
      }
      for (int i = level; i < CEILH; i++) {
        newX[i] = getPoint(i).getX() +
            appMoveRatio * (extXs[i - level] - getPoint(i).getX());
      }
      for (int i = numPoints - CEILH; i <= numPoints - 1 - level; i++) {
        newX[i] = newX[numPoints - 1 - i] + support(numPoints - 1 - i);
      }
      for (int i = 0; i < numPoints; i++) {
        points[i] = new Point(newX[i], getPoint(i).getY());
      }

      result = new PolyN(points,false);
    }
    else {
      X = getPoint(level).getX();
      extX = calExtPoint(level, LEFT);
      extXUp = calExtPointUp(level, LEFT);
      extXs = calExtPointsUp(level, LEFT);
      appMoveRatio = moveRatio * (X - extX) / (X - extXUp);
      System.out.println("X = " + X + "extX = " + extX
                         + "extXUp = " + extXUp + "appMoveRatio = "
                         + appMoveRatio);
      for (int i = 0; i < extXs.length; i++) {
        System.out.println("extXs[" + i + "] = " + extXs[i]);
      }
      for (int i = 0; i < numPoints; i++) {
        newX[i] = getPoint(i).getX();
      }
      for (int i = level; i < CEILH; i++) {
        newX[i] = getPoint(i).getX() +
            appMoveRatio * (getPoint(i).getX() - extXs[i - level]);
      }
      for (int i = numPoints - CEILH; i <= numPoints - 1 - level; i++) {
        newX[i] = newX[numPoints - 1 - i] + support(numPoints - 1 - i);
      }
      for (int i = 0; i < numPoints; i++) {
        points[i] = new Point(newX[i], getPoint(i).getY());
      }

      result = new PolyN(points,false);
    }
    //	System.out.println("result = " + result);
    if (! (result.isConvex())) {
      throw new RuntimeException("the resultant fuzzy set is non-convex");
    }
    double REPDif = this.calREP() - result.calREP();
    if (java.lang.Math.abs(REPDif) > 1e-3) {
      // System.out.println("REP before move: " + this.calREP() +
      //				  "REP after: " + result.calREP());
      throw new RuntimeException(
          "the resultant fuzzy set doesn't has the same REP");
    }
    return result;
  }

  /** move the fuzzy set with move ratios */
  public PolyN move(double[] moveRatios) throws CloneNotSupportedException {
    if (moveRatios.length != CEILH - 1) {
      throw new RuntimeException("the size of move ratios is not correct");
    }
    PolyN me = (PolyN)this.clone();
    for (int i = 0; i < CEILH - 1; i++) {
      me = me.move(moveRatios[i], i);
    }
    if (!me.isConvex()) {
      throw new RuntimeException("non-convex fuzzy set result ");
    }
    return me;
  }

  //the REP definition, note the normalized parameter is not implemented
  public double calREP() {
    double[] weights = calWeights(WeightREPType);
    double rep = 0d;
    for (int i = 0; i < numPoints; i++) {
      rep = rep + weights[i] * (getPoint(i).getX());
    }
    return rep;
  }

  /**********************************************************************/
  /******************* The REP definitions  *****************************/
  /**********************************************************************/

  public double[] calWeights(int weightIdx) {
    double[] weights = new double[numPoints];
    switch (weightIdx) {
      case 0: //the average REP definition (a0+...+a_numPoints-1)/numPoints
        for (int i = 0; i < numPoints; i++) {
          weights[i] = 1d / numPoints;
        }
        break;
      case 1: //the obsolete compatible REP definition
        for (int i = 1; i < CEILH - 1; i++) {
          weights[0] = weights[0] +
              (1d - getPoint(i).getY()) * (2d - getPoint(i).getY()) / 2d;
          weights[CEILH - 1] = weights[CEILH - 1] +
              getPoint(i).getY() * (2d - getPoint(i).getY()) / 2d;
          weights[i] = 1d / 3d - getPoint(i).getY() / 6d;
          weights[numPoints - 1 - i] = weights[i];
        }
        weights[0] = (1d - weights[0]) / 3d;
        weights[numPoints - 1] = weights[0];
        if (isOddPoints) {
          weights[CEILH - 1] = 2d * (1d / 2d - weights[CEILH - 1]) / 3d;
        }
        else {
          weights[CEILH - 1] = (1d / 2d - weights[CEILH - 1]) / 3d;
          weights[numPoints - CEILH] = weights[CEILH - 1];
        }
        break;
      // the new compatible REP definition which simply assigns
      //fake odd points as 0
      case 2:
        for (int i = 0; i < CEILH - 1; i++) {
          if(isLevelArtificiallyAdded[i]) {
            weights[i] = 0d;
          } else {
            weights[i] = (1-getPoint(i).getY()/2d)/3d;
          }
          weights[numPoints-1-i] = weights[i];
        }
        if(isOddPoints) {
          weights[CEILH -1] = 1d/3d;
        } else {
          weights[CEILH -1] = 1d/6d;
          weights[numPoints - CEILH] = weights[CEILH - 1];
        }
        break;
      case 30: //the compatible REP definition for triangular with 5 odd points
        weights = new double[5];
        weights[0] = 1d/3d;
        weights[1] = 0d;
        weights[2] = 1d/3d;
        for (int i = 3; i < 5; i++) {
            weights[i] = weights[4-i];
        }
        break;
      case 31: //as case 2 still doesn't solve the weight caluclation
        // problem of detecting isLevelArtificiallyAdded[] in construing
        // AP, APs, APm, BP, Bps and BPm, compatible REP definition for
        // hexagonal with 10 odd points is temporally used.
        weights = new double[10];
        weights[0] = 1d/3d;
        weights[1] = 0d;
        weights[2] = 1d/4d;
        weights[3] = 0d;
        weights[4] = 1d/6d;
        for (int i = 5; i < 10; i++) {
            weights[i] = weights[9-i];
        }
        break;
      //the average weighted REP definition (weights increase upwards from 1/2 to 1)
      case 4:
        for (int i = 0; i < numPoints; i++) {
          weights[i] = 0.5d + getPoint(i).getY() / 2;
        }
        break;
      case 5: //the center of the core REP definition
        for (int i = 0; i < numPoints; i++) {
          weights[i] = 0;
        }
        if (isOddPoints) {
          weights[CEILH - 1] = 1d;
        }
        else {
          weights[CEILH - 1] = 0.5d;
          weights[numPoints - CEILH] = 0.5d;
        }
        break;
    }

    return weights;
  }

  /** return the non-zero weight of the closest point upwardly */
  public double upClosestNonZeroWeight(int index) {
    if ( (index < 0) || (index > CEILH - 1)) {
      throw new RuntimeException("the index is not valid");
    }

    double[] weights = calWeights(WeightREPType);

    for (int i = index + 1; i < CEILH; i++) {
      if (weights[i] != 0) {
        return weights[i];
      }
    }
    return 0;
  }

  /** test if the fuzzy set is convex */
  public boolean isConvex() {
    for (int i = 0; i < numPoints - 1; i++) {
      if (getPoint(i).getX() - getPoint(i + 1).getX() > 1e-3) {
        return false;
      }
    }
    return true;
  }

  /** test if the fuzzy set has paired odd points */
  public boolean isPaired() {
    double dif;
    for (int i = 0; i < CEILH; i++) {
      dif = points[i].getY() - points[numPoints - 1 - i].getY();
      if (java.lang.Math.abs(dif) > 1e-3) {
        return false;
      }
    }
    return true;
  }

  /** test if two fuzzy sets have the same positions on equal
   * and below the specified level
   */
  public boolean isEqualUnder(PolyN p, int level) {
    for (int i = 0; i < level; i++) {
      if (! (getPoint(i).isEqual(p.getPoint(i)))) {
        return false;
      }
      if (! (getPoint(numPoints - 1 - i).isEqual(p.getPoint(numPoints - 1 - i)))) {
        return false;
      }
    }
    return true;
  }

  public final int ceilHalf(int numPoints) {
    if (numPoints % 2 == 0) {
      return numPoints / 2;
    }
    else {
      return (numPoints + 1) / 2;
    }
  }

  public final int floorHalf(int numPoints) {
    if (numPoints % 2 == 0) {
      return numPoints / 2;
    }
    else {
      return (numPoints - 1) / 2;
    }
  }

  /** print all artificial levels the fuzzy set has */
  public void printArtificialLevels() {
    for(int i = 0; i < CEILH-1; i++) {
      if(isLevelArtificial(i)) {
        System.out.println("alpha cut " + i + " with membership value "
                           + getPoint(i).getY() + " is artificial");
      }
    }
  }

  /** test if the level is artificial, i.e., the two points lying on this
   * alpha cut are artificial or not */
  public boolean isLevelArtificial(int pointIdx) {
    if((pointIdx < 0)||(pointIdx > CEILH-1)) {
      throw new RuntimeException("point index is not valid");
    }

    if((pointIdx == 0)||(pointIdx == CEILH-1)) {
      return false;
    }

    boolean leftPointArtificial = false;
    if((getPoint(pointIdx).getX()==getPoint(pointIdx+1).getX())&&
       (getPoint(pointIdx).getX()==getPoint(pointIdx-1).getX())) {
      leftPointArtificial = true;
    } else {
      double XRatioLeft = (getPoint(pointIdx).getX()-getPoint(pointIdx-1).getX())/
          (getPoint(pointIdx+1).getX()-getPoint(pointIdx).getX());
      double YRatioLeft = (getPoint(pointIdx).getY()-getPoint(pointIdx-1).getY())/
          (getPoint(pointIdx+1).getY()-getPoint(pointIdx).getY());
      if(java.lang.Math.abs(XRatioLeft-YRatioLeft) < 1e-5) {
        leftPointArtificial = true;
      }
    }
   // System.out.println("leftPointArtificial = " + leftPointArtificial);

    boolean rightPointArtificial = false;
    if((getPoint(numPoints-1-pointIdx).getX()==getPoint(numPoints-2-pointIdx).getX())&&
       (getPoint(numPoints-1-pointIdx).getX()==getPoint(numPoints-pointIdx).getX())) {
      rightPointArtificial = true;
    } else {
      double XRatioRight = (getPoint(numPoints-1-pointIdx).getX()-getPoint(numPoints-2-pointIdx).getX())/
         (getPoint(numPoints-pointIdx).getX()-getPoint(numPoints-1-pointIdx).getX());
     double YRatioRight = (getPoint(numPoints-2-pointIdx).getY()-getPoint(numPoints-1-pointIdx).getY())/
         (getPoint(numPoints-1-pointIdx).getY()-getPoint(numPoints-pointIdx).getY());
      if(java.lang.Math.abs(XRatioRight-YRatioRight) < 1e-5) {
        rightPointArtificial = true;
      }
    }
   // System.out.println("rightPointArtificial = " + rightPointArtificial);
    if(leftPointArtificial&&rightPointArtificial) {
      return true;
    } else {
      return false;
    }
  }

  /** print the fuzzy set in a readable form */
  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("Poly(" + numPoints + "): ");
    for (int i = 0; i < numPoints; i++) {
      sb.append(new PrintfFormat("%3.2f").sprintf(getPoint(i).getX()) + " ");
      //sb.append(getPoint(i).getX() + " ");
    }
    return sb.toString();
  }

}
