package huang;

import java.io.*;
import java.awt.*;
import javax.swing.JFrame;

public class PolyNInterpolation extends javax.swing.JPanel {
  //fields initiated in the construct of the class
  private PolyN A1, A2, AS, B1, B2;
  //fields initiated in the interpolation method
  private PolyN AP, BP, APm, BPm;
  private int NPoints;
  final int LEFT = 0;
  final int RIGHT = 1;  
  final int UP = 2;
  final int DOWN = 3;
  final double SIN30 = 0.5d;
  final double COS30 = java.lang.Math.sqrt(3d) / 2d;

  final int SCALE_ORIGINAL = 0;
  final int SCALE_LINEAR = 1;
  final int MOVE_ORIGINAL = 2;
  final int MOVE_LINEAR = 3;
  /************************ options ***************************/
  public int scaleMethod = 0;
  public int moveMethod = 2;

  /** construct method */
  public PolyNInterpolation(PolyN A1, PolyN A2, PolyN AS,
                            PolyN B1, PolyN B2) {
    this.A1 = A1;
    this.A2 = A2;
    this.AS = AS;
    this.B1 = B1;
    this.B2 = B2;
    NPoints = A1.getNumPoints();
  }

  /** read from a file to initiate A1, A2, AS, B1 and B2*/
  public PolyNInterpolation(Reader r) throws IOException {
    StreamTokenizer st = new StreamTokenizer(r);
    double val1, val2;
    Point[] points;

    st.whitespaceChars(',', ',');
    st.whitespaceChars(';', ';');
    st.commentChar('%');
    st.quoteChar('"');
    st.quoteChar('\'');
    st.eolIsSignificant(false);

    while (st.nextToken() != StreamTokenizer.TT_EOF) {
      int n = (int) st.nval;
      points = new Point[n];
      // NPoints = n;
      String[] fuzzySetNames = {
          "A1", "A2", "AS", "B1", "B2"};
      st.nextToken();
      for (int j = 0; j < 5; j++) {
        System.out.println("j = " + j);
        if (! (st.sval.equalsIgnoreCase(fuzzySetNames[j]))) {
          System.err.println(fuzzySetNames[j] + " is expected");
        }
        for (int i = 0; i < n; i++) {
          st.nextToken();
          val1 = st.nval;
          //System.out.println("val1 = " + val1);
          st.nextToken();
          val2 = st.nval;
          points[i] = new Point(val1, val2);
          System.out.println(points[i]);
        }
        //System.out.println("n = " + n);
        switch (j) {
        case 0:
          A1 = new PolyN(points, true);
          break;
        case 1:
          A2 = new PolyN(points, true);
          break;
        case 2:
          AS = new PolyN(points, true);
          break;
        case 3:
          B1 = new PolyN(points, true);
          break;
        case 4:
          B2 = new PolyN(points, true);
          break;
        }
        st.nextToken();
      }
      NPoints = A1.getNumPoints();
    }
  }

  /** return the fuzzy set via KH interpolation */
  public void KHInterpolation() {
    //below comes the KH interpolation
    System.out.println("*****************************************");
    //System.out.println("AS =" + AS);
    Point[] a1 = A1.getPoints();
    Point[] a2 = A2.getPoints();
    Point[] b1 = B1.getPoints();
    Point[] b2 = B2.getPoints();
    Point[] as = AS.getPoints();
    Point[] bs = new Point[NPoints];
    double[] lam = new double[NPoints];
    PolyN result;

    //System.out.println("NPoints = " + NPoints);
    for (int i = 0; i < NPoints; i++) {
      lam[i] = as[i].calLambda(a1[i], a2[i]);
      bs[i] = (b1[i].multiply(1 - lam[i])).add(b2[i].multiply(lam[i]));
    }

    result = new PolyN(bs, false);
    System.out.println("the KH interpolation result:\n" + result);
  }

  /** return the fuzzy set (only in triangular case) via Peter's general
   * interpolation */
  public void GeneralInterpolation() {
    if (NPoints != 3) {
      throw new RuntimeException("the general interpolation method is" +
                                 " only implemented for the triangular cases");
    }
    System.out.println("\n*****************************************");
    System.out.println("the General interpolation result:");
    System.out.println("A1 = " + A1);
    System.out.println("A2 = " + A2);
    System.out.println("AS = " + AS);

    //use the middle point to calculate the lambda
    double lam = AS.getPoint(1).calLambda(A1.getPoint(1), A2.getPoint(1));
    System.out.println("lambda is " + lam);
    AP = lambdaInt(A1, A2, lam); //note AP is a global variable
    System.out.println("AP: " + AP);
    BP = lambdaInt(B1, B2, lam); //note BP is a global variable
    System.out.println("BP: " + BP);

    if (!AP.getPoint(1).isEqual(AS.getPoint(1))) {
      throw new RuntimeException("AP's middle point is not equals to AS's");
    }

    Point[] ap = AP.getPoints();
    Point[] bp = BP.getPoints();
    Point[] as = AS.getPoints();
    Point[] bs = new Point[NPoints];
    double leftRatio, rightRatio;
    Point tempAP1 = new Point(ap[1].getX(), 0d);
    Point tempBP1 = new Point(bp[1].getX(), 0d);
    Point pointMinValA = new Point(minValA(), 0d);
    Point pointMaxValA = new Point(maxValA(), 0d);
    Point pointMinValB = new Point(minValB(), 0d);
    Point pointMaxValB = new Point(maxValB(), 0d);

    System.out.println("calculate the left point:");
    if (ap[0].getX() <= as[0].getX()) { //shrink
      leftRatio = as[0].calLambda(ap[0], tempAP1);
      System.out.println("shrink: leftRatio = " + leftRatio);
      bs[0] = bp[0].calInterpolation(tempBP1, leftRatio);
    } else { //expand
      leftRatio = as[0].calLambda(pointMinValA, ap[0]);
      System.out.println("expand: leftRatio = " + leftRatio);
      bs[0] = pointMinValB.calInterpolation(bp[0], leftRatio);
    }
    System.out.println("bs[0] = " + bs[0]);

    System.out.println("calculate the right point:");
    if (ap[2].getX() >= as[2].getX()) { //shrink
      rightRatio = as[2].calLambda(tempAP1, ap[2]);
      System.out.println("shrink: rightRatio = " + rightRatio);
      bs[2] = tempBP1.calInterpolation(bp[2], rightRatio);
    } else {
      rightRatio = as[2].calLambda(ap[2], pointMaxValA);
      System.out.println("expand: rightRatio = " + rightRatio);
      bs[2] = bp[2].calInterpolation(pointMaxValB, rightRatio);
      System.out.println("pointMaxValA = "+ pointMaxValA);
      System.out.println("pointMaxValB = "+ pointMaxValB);
    }
    System.out.println("bs[2] = " + bs[2]);
    bs[1] = bp[1];

    PolyN result = new PolyN(bs, false);
    System.out.println("the General interpolation result:\n" + result);
  }

  /** return the fuzzy set (only in triangular case) via QMY
  * interpolation */
 public void QMYInterpolation() {
   if (NPoints != 3) {
     throw new RuntimeException("the QMY interpolation method is" +
                                " only implemented for the triangular cases");
   }
   System.out.println("\n*****************************************");
   System.out.println("the QMY interpolation result:");
   System.out.println("A1 = " + A1);
   System.out.println("A2 = " + A2);
   System.out.println("AS = " + AS);

   //use the middle point to calculate the lambda
   double lam = AS.getPoint(1).calLambda(A1.getPoint(1), A2.getPoint(1));
   System.out.println("lambda is " + lam);
   AP = lambdaInt(A1, A2, lam); //note AP is a global variable
   System.out.println("AP: " + AP);
   BP = lambdaInt(B1, B2, lam); //note BP is a global variable
   System.out.println("BP: " + BP);

   if (!AP.getPoint(1).isEqual(AS.getPoint(1))) {
     throw new RuntimeException("AP's middle point is not equals to AS's");
   }

   Point[] ap = AP.getPoints();
   Point[] bp = BP.getPoints();
   Point[] as = AS.getPoints();
   Point[] bs = new Point[NPoints];
   double leftRatio, rightRatio;

   System.out.println("calculate the left point:");

   leftRatio = (as[1].getX()-as[0].getX())/(ap[1].getX()-ap[0].getX());
   System.out.println("leftRatio = " + leftRatio);
   double x0 = bp[1].getX()-leftRatio*(bp[1].getX()-bp[0].getX());
   bs[0] = new Point(x0, 0d);
   System.out.println("bs[0] = " + bs[0]);

   System.out.println("calculate the right point:");
   rightRatio = (as[2].getX()-as[1].getX())/(ap[2].getX()-ap[1].getX());
   System.out.println("rightRatio = " + rightRatio);
   double x2 = rightRatio*(bp[2].getX()-bp[1].getX())+bp[1].getX();
   bs[2] = new Point(x2, 0d);
   System.out.println("bs[2] = " + bs[2]);
   bs[1] = bp[1];

   PolyN result = new PolyN(bs, false);
   System.out.println("the General interpolation result:\n" + result);
 }


  /** apply HS interpolation, working out AP, BP, APm, BPm */
  public void HSInterpolation() {
    int ceilH = A1.ceilHalf(NPoints);
    int floorH = A1.floorHalf(NPoints);

    double lam;
    double[] scaleRateA = new double[floorH];
    double[] scaleRateB = new double[floorH];
    double[] scaleRatios = new double[floorH];
    double[] moveRatios = new double[ceilH - 1];

    //below comes the HS interpolation
    System.out.println("\n*****************************************");
    System.out.println("the HS interpolation result:");
    System.out.println("A1 = " + A1);
    System.out.println("A2 = " + A2);
    System.out.println("AS = " + AS);

    System.out.println("A1.calREP() = " + A1.calREP());
    System.out.println("A2.calREP() = " + A2.calREP());
    System.out.println("AS.calREP() = " + AS.calREP());
    lam = calLambda();
    System.out.println("lambda is " + lam);
    AP = lambdaInt(A1, A2, lam);
    System.out.println("AP: " + AP);
    System.out.println("AP.calREP() = " + AP.calREP());
    AP.printArtificialLevels();
    BP = lambdaInt(B1, B2, lam);
    System.out.println("BP: " + BP);
    BP.printArtificialLevels();

    //scale transformations
    System.out.println("\nScale transformation:");
    for (int i = 0; i < floorH; i++) {
      scaleRateA[i] = AS.support(i) / AP.support(i);
      System.out.println(i + "-th support scale rate of A = " + scaleRateA[i]);
    }

    if (scaleMethod == SCALE_ORIGINAL) {
      scaleRatios = AP.calS(AS);
      for (int i = 0; i < floorH; i++) {
        System.out.println(i + "-th support scale ratio = " + scaleRatios[i]);
      }

      scaleRateB = BP.cals(scaleRateA[0], scaleRatios);
      for (int i = 0; i < floorH; i++) {
        System.out.println(i + "-th support scale rate of B  = " + scaleRateB[i]);
      }
    } else if (scaleMethod == SCALE_LINEAR) {
      double[] scaleCriterias = AP.calSLinear(AS);
      for (int i = 0; i < NPoints; i++) {
        System.out.println(i + "-th scale criteria = " + scaleCriterias[i]);
      }

      scaleRateB = BP.calsLinear(scaleRateA[floorH - 1], scaleCriterias);
      for (int i = 0; i < floorH; i++) {
        System.out.println(i + "-th support scale rate of B  = " + scaleRateB[i]);
      }
    }

    PolyN APs = AP.scale(scaleRateA);
    System.out.println("APs: " + APs);
    APs.printArtificialLevels();
    PolyN BPs = BP.scale(scaleRateB);
    System.out.println("BPs: " + BPs);
    BPs.printArtificialLevels();
    System.out.println();

    //Move transformations
    System.out.println("Move transformation:");
    APm = APs;
    BPm = BPs;
    double[] weights = APm.calWeights(APm.WeightREPType);
    for (int i = 0; i < weights.length; i++) {
      System.out.println("weights[" + i + "] = " + weights[i]);
    }
    System.out.println("APm = " + APm);
    System.out.println("AS = " + AS);
    System.out.println("APm.calREP() = " + APm.calREP());
    System.out.println("AS.calREP() = " + AS.calREP());
    if (moveMethod == MOVE_ORIGINAL) {
      for (int i = 0; i < ceilH - 1; i++) {
        moveRatios[i] = APm.calM(AS, i);
        System.out.println(i + "-th move ratio = " + moveRatios[i]);
        APm = APm.move(moveRatios[i], i);
        System.out.println(i + "-th APm: " + APm);
        APm.printArtificialLevels();
        BPm = BPm.move(moveRatios[i], i);
        System.out.println(i + "-th BPm: " + BPm);
        BPm.printArtificialLevels();
        //less sub-moves may be applied(e.g., only one is needed for the
        //interpolation of triagular sets but each with n odd points, n >3)
        if (APm.isEqualUnder(AS, ceilH)) {
          break;
        }
      }
      if (! (APm.isEqualUnder(AS, ceilH))) {
        throw new RuntimeException("total move is not consistent!");
      }
    } else if (moveMethod == MOVE_LINEAR) {
      double[] moveCriterias = APm.calMLinear(AS);
      for (int i = 0; i < moveCriterias.length; i++) {
        System.out.println("moveCriterias[" + i + "] = " + moveCriterias[i]);
      }
      APm = APm.moveLinear(moveCriterias);
      System.out.println("APm: " + APm);
      System.out.println("APm.calREP() = " + APm.calREP());
      APm.printArtificialLevels();
      if (! (APm.isEqualUnder(AS, ceilH))) {
        throw new RuntimeException("linear move cannot move to AS!");
      }
      BPm = BPm.moveLinear(moveCriterias);
      System.out.println("BPm: " + BPm);
      System.out.println("BPm.calREP() = " + BPm.calREP());
      BPm.printArtificialLevels();
    }
  }

  /** calculate the lambda of the antecedent part */
  public double calLambda() {
    double REP1, REP2, REP;
    double dis1, dis2;
    REP1 = A1.calREP();
    //System.out.println("REP1 = " + REP1);
    REP2 = A2.calREP();
    REP = AS.calREP();
    dis1 = REP - REP1;
    //System.out.println("dis1 = " + dis1);
    dis2 = REP2 - REP1;
    //System.out.println("dis2 = " + dis2);
    return dis1 / dis2;
  }

  /** Use the lambda to interpolate two fuzzy sets */
  public PolyN lambdaInt(PolyN p1, PolyN p2, double lambda) {
    Point[] t1 = p1.getPoints();
    Point[] t2 = p2.getPoints();
    Point[] t = new Point[t1.length];
    PolyN result;
    for (int i = 0; i < t1.length; i++) {
      t[i] = (t1[i].multiply(1 - lambda)).add(t2[i].multiply(lambda));
    }
    result = new PolyN(t, false);
    if (!result.isConvex()) {
      throw new RuntimeException("non-convex PolyN ");
    }
    return result;
  }

  /** get the min value of the x input, this method is only invoked
   * by General interpolation, which is imeplemented based on triagular
   * cases */
  public double minValA() {
    return A1.getPoint(0).getX();
  }

  /** get the max value of the x input, this method is only invoked
   * by General interpolation, which is imeplemented based on triagular
   * cases */
  public double maxValA() {
    return A2.getPoint(NPoints-1).getX();
  }

  /** get the min value of the y input, this method is only invoked
   * by General interpolation, which is imeplemented based on triagular
   * cases */
  public double minValB() {
    return B1.getPoint(0).getX();
  }

  /** get the max value of the y input, this method is only invoked
   * by General interpolation, which is imeplemented based on triagular
   * cases */
  public double maxValB() {
    return B2.getPoint(NPoints-1).getX();
  }

  /*********************************************************************
   **************************  GUI stuff *******************************
   *********************************************************************/

  /** paint the fuzzy interpolation */
  public void paint(Graphics g) {
    super.paint(g);
    //g.setColor(Color.BLACK);
    drawResults(g);
  }

  /** after the invokation of interpolation, AP, BP, APm, BPm
   * are worked out so that this method can draw them
   */
  public void drawResults(Graphics g) {
    // draw the fuzzy sets
    setBackground(Color.WHITE);
    CorDraw(Color.BLACK, g);
    PolyNDraw(A1, 0, Color.BLACK, g);
    PolyNDraw(A2, 0, Color.BLACK, g);
    PolyNDraw(AP, 0, Color.BLUE, g);
    //PolyNDraw(APs, 0, Color.BLUE, g);
    PolyNDraw(AS, 0, Color.BLACK, g);
    PolyNDraw(B1, 1, Color.BLACK, g);
    PolyNDraw(B2, 1, Color.BLACK, g);
    PolyNDraw(BP, 1, Color.BLUE, g);
    //PolyNDraw(BPs, 1, Color.BLUE, g);
    PolyNDraw(BPm, 1, Color.RED, g);
  }

  /** work out the scales rates to draw the fuzzy sets */
  public double[] scaleOfCor() {
    double[] scales = new double[2];
    double Xmax = A1.getPoint(NPoints - 1).getX();
    double width = getSize().width;
    double height = getSize().height;
    if (Xmax < AS.getPoint(NPoints - 1).getX()) {
      Xmax = AS.getPoint(NPoints - 1).getX();
    }
    if (Xmax < A2.getPoint(NPoints - 1).getX()) {
      Xmax = A2.getPoint(NPoints - 1).getX();
    }
    if (Xmax < B1.getPoint(NPoints - 1).getX()) {
      Xmax = B1.getPoint(NPoints - 1).getX();
    }
    if (Xmax < B2.getPoint(NPoints - 1).getX()) {
      Xmax = B2.getPoint(NPoints - 1).getX();
    }
    scales[0] = 0.8d * width / Xmax;
    scales[1] = (1d / 2d) * 0.7d * height / 1d;
    return scales;
  }

  /** draw the coordinates */
  public void CorDraw(Color c, Graphics g) {
    double[] scalesCor = scaleOfCor();
    double Xscale = scalesCor[0];
    double Yscale = scalesCor[1];
    double width = getSize().width;
    double height = getSize().height;
    int maxVal = (int) (0.8 * width / Xscale);
    System.out.println(maxVal);
    g.setColor(c);
    g.drawLine( (int) (0.1 * width), (int) (0.45 * height), (int) (0.95 * width),
               (int) (0.45 * height));
    drawArrow( (int) (0.95 * width), (int) (0.45 * height), RIGHT, 10d, g);
    g.drawLine( (int) (0.1 * width), (int) (0.45 * height), (int) (0.1 * width),
               (int) (0.05 * height));
    drawArrow( (int) (0.1 * width), (int) (0.05 * height), UP, 10d, g);

    g.drawLine( (int) (0.1 * width), (int) (0.9 * height), (int) (0.95 * width),
               (int) (0.9 * height));
    drawArrow( (int) (0.95 * width), (int) (0.9 * height), RIGHT, 10d, g);
    g.drawLine( (int) (0.1 * width), (int) (0.9 * height), (int) (0.1 * width),
               (int) (0.5 * height));
    drawArrow( (int) (0.1 * width), (int) (0.5 * height), UP, 10d, g);

    g.setFont(new Font("serif", Font.PLAIN, 12));
    for (int i = 0; i <= maxVal; i = i + 1) {
      g.drawLine( (int) (0.1 * width + i * Xscale), (int) (0.45 * height),
                 (int) (0.1 * width + i * Xscale), (int) (0.435 * height));
      g.drawString(String.valueOf(i), (int) (0.09 * width + i * Xscale),
                   (int) (0.49 * height));
      g.drawLine( (int) (0.1 * width + i * Xscale), (int) (0.9 * height),
                 (int) (0.1 * width + i * Xscale), (int) (0.885 * height));
      g.drawString(String.valueOf(i), (int) (0.09 * width + i * Xscale),
                   (int) (0.94 * height));
    }
  }

  /** draw the arrows */
  public void drawArrow(int x, int y, int dir, double len, Graphics g) {
    int xVal, yVal;
    switch (dir) {
    case LEFT: {
      xVal = x + (int) (len * COS30);
      yVal = y + (int) (len * SIN30);
      g.drawLine(x, y, xVal, yVal);
      yVal = y - (int) (len * SIN30);
      g.drawLine(x, y, xVal, yVal);
      break;
    }
    case RIGHT: {
      xVal = x - (int) (len * COS30);
      yVal = y + (int) (len * SIN30);
      g.drawLine(x, y, xVal, yVal);
      yVal = y - (int) (len * SIN30);
      g.drawLine(x, y, xVal, yVal);
      break;
    }
    case UP: {
      xVal = x + (int) (len * SIN30);
      yVal = y + (int) (len * COS30);
      g.drawLine(x, y, xVal, yVal);
      xVal = x - (int) (len * SIN30);
      g.drawLine(x, y, xVal, yVal);
      break;
    }
    case DOWN: {
      xVal = x - (int) (len * SIN30);
      yVal = y - (int) (len * COS30);
      g.drawLine(x, y, xVal, yVal);
      xVal = x + (int) (len * SIN30);
      g.drawLine(x, y, xVal, yVal);
      break;
    }
    default: {}
    }
  }

  /** draw the fuzzy sets */
  public void PolyNDraw(PolyN p, double Yoffset, Color c, Graphics g) {
    double[] scalesCor = scaleOfCor();
    double Xscale = scalesCor[0];
    double Yscale = scalesCor[1];
    double width = getSize().width;
    double height = getSize().height;
    g.setColor(c);
    for (int i = 0; i < NPoints - 1; i++) {
      g.drawLine( (int) (0.1d * width + (p.getPoint(i).getX()) * Xscale),
                 (int) (height -
                        (0.1d * height +
                         (p.getPoint(i).getY() + (0.45 / 0.35) * (1 - Yoffset)) *
                         Yscale)),
                 (int) (0.1d * width + (p.getPoint(i + 1).getX()) * Xscale),
                 (int) (height -
                        (0.1d * height +
                         (p.getPoint(i + 1).getY() +
                          (0.45 / 0.35) * (1 - Yoffset)) * Yscale)));
    }
  }

public static void main(String[] args) throws Exception {
  //String fileName = "interp_data_hex_new";  
  //String fileName = "interp_data_trape";
  //String fileName = "interp_data_hex_10_points";
  //String fileName = "interp_data_trian";
  String fileName = "Example56";
  Reader r = new BufferedReader(new FileReader(fileName));
  PolyNInterpolation pni = new PolyNInterpolation(r);

  pni.KHInterpolation();
  //pni.GeneralInterpolation(); // only implemented in triangular cases
  //pni.QMYInterpolation(); // only implemented in triangular cases
  pni.HSInterpolation();
  JFrame a = new JFrame("Demo of Interpolation Developed by Longzhi@Aberystwyth");
  a.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
  a.setSize(800, 600);
  a.setVisible(true);
  a.getContentPane().add(pni,BorderLayout.CENTER);
  pni.setSize(800,600);
//  a.pack();
  }

}
