package huang.inequalitiesSolver;
import java.util.ArrayList;

public class Problem {

	private ArrayList<Variable> variables;
	private ArrayList<Edge> edges;

	public Problem() {
		variables = new ArrayList<Variable>();
		edges = new ArrayList<Edge>();
	}

	public void addEdge(Edge edge) {
		boolean flag1 = false;
		boolean flag2 = false;
		for (Variable v : variables) {
			if (v.equals(edge.getStarter()))
				flag1 = true;
			if (v.equals(edge.getEnder()))
				flag2 = true;
		}
		if (flag1 == false)
			variables.add(edge.getStarter());
		if (flag2 == false)
			variables.add(edge.getEnder());
		edges.add(edge);
	}

	public ArrayList<Edge> getEdges() {
		return edges;
	}

	public ArrayList<Variable> getVariables() {
		return variables;
	}

	public boolean solve() {
		for(Variable v: variables){
			System.out.println("Start to solve the problem. Variable is: v" + v.getID());
		}
		if (preSolve() == false)
			return false;
		System.out.println("After tiding up, " +printProblem());
		for (Edge edge1 : edges) {
			for (Edge edge2 : edges) {
				int i = edge1.variablesUnion(edge2);
				System.out.println("i is: " + i);
				// if(i<9) ;
				// else
				if (edge1.restrictSolution(i, edge2) == false) {
					return false;
				}

			}

		}
		return true;
	}

	public boolean preSolve() {
		boolean flag = true;
		System.out.println("This is presolve precedure.");
		for (int l = edges.size() - 1; l >= 0; l--) {
			Edge edge = edges.get(l);
			if (edge.getA() == 0) {
				if (edge.getB() == 0) {
					if (edge.getC() < 0)
						return false;
					else {
						edges.remove(edge);
					}
				} else {
					if (edge.getB() < 0) {
						double min = edge.getC() / edge.getB();
						flag = flag && edge.getEnder().setMin(min);
						edges.remove(edge);
					} else {
						double max = edge.getC() / edge.getB();
						flag = flag &&edge.getEnder().setMax(max);
						edges.remove(edge);
					}
				}
			}

			else if (edge.getB() == 0) {
				if (edge.getA() < 0) {
					double min = edge.getC() / edge.getA();
					flag = flag &&edge.getStarter().setMin(min);
					edges.remove(edge);
				} else {
					double max = edge.getC() / edge.getA();
					flag = flag &&edge.getStarter().setMax(max);
					edges.remove(edge);
				}
			}
		}

		System.out.println("There are "+variables.size()+" variables.");
		for(Variable v: variables){
			System.out.println("Variable is: v" + v.getID());
		}
		for (Variable v : variables) { // need to revise for multi-variables
			for (Variable v2 : variables) {
				if (!v2.equals(v)) {
					Edge edge;
					if (v.flagMin != 0)
						edge = new Edge(v, -1, v2, 0, -v.getMin());
					else
						edge = new Edge(v, -1, v2, 0, Float.POSITIVE_INFINITY);
					edges.add(edge);
					v.minEdges.add(edge);
				}
			}

			for (Variable v2 : variables) {
				if (v2 != v) {
					Edge edge;
					if (v.flagMax != 0)
						edge = new Edge(v, 1, v2, 0, v.getMax());
					else
						edge = new Edge(v, 1, v2, 0, Float.POSITIVE_INFINITY);
					edges.add(edge);
					v.maxEdges.add(edge);
				}
			}

		}
		return flag;
	}

	public String printProblem() {
		String s = "The given problem inequations are: \n";
		for (Edge e : edges) {
			if (e.getB() < 0)
				s += e.getA() + "v" + e.getStarter().getID() + " " + e.getB()
						+ "v" + e.getEnder().getID() + " <= " + e.getC() + "\n";
			else
				s += e.getA() + " v" + e.getStarter().getID() + " +" + e.getB()
						+ "v" + e.getEnder().getID() + " <= " + e.getC() + "\n";
		}
		return s;
	}

	public String printSolution() {
		String s;
		s = "The solution of the problem is: \n";
		for (Variable v : variables) {
			if (v.flagMin != 0 && v.flagMax != 0)
				s += v.getMin() + " <= " + "v" + v.getID() + " <= "
						+ v.getMax() + "\n";
			else {
				if (v.flagMin != 0 && v.flagMax != 0)
					s += "v" + v.getID() + " >= " + v.getMin() + "\n";
				if (v.flagMax != 0)
					s += "v" + v.getID() + " <= " + v.getMax() + "\n";
			}
		}
		return s;
	}
}
